#!/usr/bin/env python
#coding:utf-8
# Author        : tuxpy
# Email         : q8886888@qq.com.com
# Last modified : 2015-06-04 11:28:26
# Filename      : session.py
# Description   : 
from __future__ import unicode_literals, print_function

from gale.escape import utf8

import uuid
import threading
import hmac
import json

import hashlib
import os
import time

class SessionData(dict):
    def __init__(self, session_id, hmac_key):
        self.session_id = session_id
        self.hmac_key = hmac_key
        dict.__init__(self)

class Session(SessionData):
    def __init__(self, session_manager, request_handler):
        self.session_manager = session_manager
        self.request_handler = request_handler

        try:
            current_session = session_manager.get(request_handler)
        except InvalidSessionException:
            current_session = session_manager.get()

        for key, data in current_session.items():
            self[key] = data

        self.session_id = current_session.session_id
        self.hmac_key = current_session.hmac_key

        dict.__init__(self)

    def __getattr__(self, name):
        if name not in self:
            raise AttributeError

        return self[name]

    def save(self):
        self.session_manager.set(self.request_handler, self)

    def flush(self):
        self.session_manager.flush(self.request_handler, self)

class ISessionManager(object):
    def __init__(self):
        raise ImportError

    def get(self, request_handler = None):
        session_id = (request_handler and request_handler.get_signed_cookie(self.session_id_key)) or self._generate_id()
        hmac_key = (request_handler and request_handler.get_signed_cookie(self.session_verify_key)) or self._generate_hmac(session_id)

        check_hmac = self._generate_hmac(session_id)
        if hmac_key != check_hmac:
            raise InvalidSessionException

        session = SessionData(session_id, hmac_key)
        session_data = self._fetch(session_id)
        session.update(session_data)

        return session

    def _fetch(self, session_id):
        raise NotImplementedError

    def set(self, request_handler, session):
        request_handler.set_signed_cookie(self.session_id_key, session.session_id, expires_day = 30)
        request_handler.set_signed_cookie(self.session_verify_key, session.hmac_key, expires_day = 30)
        self.save_session(session)

    def flush(self, request_handler, session):
        request_handler.clear_cookie(self.session_id_key)
        request_handler.clear_cookie(self.session_verify_key)
        self.remove_session(session)

    def save_session(self, session):
        raise NotImplementedError

    def remove_session(self, session):
        raise NotImplementedError

    def _generate_id(self):
        _id = 'GALE%s' % hashlib.sha1(utf8(self.session_secret) + utf8(str(uuid.uuid4()))).hexdigest()
        return _id

    def _generate_hmac(self, session_id):
        return hmac.new(utf8(session_id), utf8(self.session_secret), hashlib.sha1).hexdigest()

class FileSessionManager(ISessionManager):
    """这是使用文件来实现的seession机制"""
    lock = threading.RLock()

    def __init__(self, session_secret, session_timeout, session_path = None, session_id_key = 'SESSION_ID', session_verify_key = 'VERIFICATION'):
        self.session_secret = session_secret
        self.session_timeout = session_timeout
        self.session_id_key = session_id_key
        self.session_verify_key = session_verify_key

        import tempfile
        session_path = session_path or tempfile.gettempdir()
        if not os.path.isdir(session_path):
            raise SessionPathNotExists('session path not exist')

        self.session_path = session_path
        self.SESSION_PREFIX = '_glab_sess_'

        from gale.utils import ShareDict

        self.__file_map = ShareDict() # 文件的访问时间记录到一张hash表中去
        self.__made_file_map(self.__file_map)

    def __made_file_map(self, file_map):
        import glob
        for session_file in glob.glob(os.path.join(self.session_path, self.SESSION_PREFIX) + '*'):
            _stat = os.stat(session_file)
            if time.time() > _stat.st_atime + self.session_timeout:
                os.remove(session_file)
            else:
                file_map[session_file] = _stat.st_atime

    def __update_file_map(self, session_file):
        for session_file, session_atime in self.__file_map.items():
            if time.time()  > session_atime + self.session_timeout:
                os.remove(session_file)
                del self.__file_map[session_file]

        self.__file_map[session_file] = time.time() # 把刚访问的那个文件在表中的访问日期也更新掉

    def _fetch(self, session_id):
        session_file = os.path.join(self.session_path, 
                '%s%s' % (self.SESSION_PREFIX, session_id))

        self.__update_file_map(session_file) # 每次有访问信息进来的，就去检查下hash表

        try:
            with open(session_file, 'rb') as session_fd:
                session_data = json.loads(session_fd.read())
        except IOError:
            return {}

        return isinstance(session_data, dict) and session_data or {}

    def save_session(self, session):
        session_file = os.path.join(self.session_path, 
                '%s%s' % (self.SESSION_PREFIX, session.session_id))
        session_data = json.dumps(dict(session.items())) 

        with self.lock:
            with open(session_file, 'wb') as session_fd:
                session_fd.write(session_data)

    def remove_session(self, session):
        session_file = os.path.join(self.session_path, 
                '%s%s' % (self.SESSION_PREFIX, session.session_id))
        del session
        if os.path.exists(session_file):
            os.remove(session_file)

class RedisSessionManager(ISessionManager):
    def __init__(self, session_secret, session_timeout, session_db, 
            session_id_key = 'SESSION_ID', session_verify_key = 'VERIFICATION'):
        self.session_secret = session_secret
        self.session_timeout = session_timeout
        self.session_db = session_db
        self.session_id_key = session_id_key
        self.session_verify_key = session_verify_key

    def _fetch(self, session_id):
        raw_data = self.session_db.get(session_id)
        if not raw_data:
            return {}

        self.session_db.expire(session_id, self.session_timeout)

        session_data = json.loads(raw_data)
        return isinstance(session_data, dict) and session_data or {}

    def save_session(self, session):
        session_data = json.dumps(dict(session.items()))
        self.session_db.setex(session.session_id, session_data, self.session_timeout)

    def remove_session(self, session):
        self.session_db.delete(session.session_id)

class SessionPathNotExists(Exception):
    pass

class InvalidSessionException(Exception):
    pass




#!/usr/bin/env python
#coding:utf8
# Author          : tuxpy
# Email           : q8886888@qq.com
# Last modified   : 2015-03-26 13:14:37
# Filename        : gale/server.py
# Description     : 
from __future__ import unicode_literals, print_function

try:
    from gale.stream import StreamServer
except ImportError as e:
    from gale.wsgi.stream import StreamServer

from gale.ipc import IPCServer
import atexit

from multiprocessing import  cpu_count, Process

class ServerSettings(dict):
    def __new__(self, *args, **kwargs):
        if hasattr(ServerSettings, '_instance'):
            return ServerSettings._instance
        instance = dict.__new__(self, *args, **kwargs)
        ServerSettings._instance = instance

        return ServerSettings._instance

    def __setattr__(self, name, value):
        self[name] = value

    def __getattr__(self, name):
        try:
             return self[name]
        except KeyError:
             raise

class HTTPServer(object):
    """
        http server
    """
    def __init__(self, callback, host = '0.0.0.0', port = 8080, timeout = 15, max_client = 1000, autoreload = True,
            load_worker = None):
        assert callable(callback) # callback必须是可调用的
        self.host = host
        self.port = port
        self._callback = callback # 表示回调函数
        self.autoreload = autoreload
        self.run_pool = []
        if load_worker: # 如果有load worker则需要将它连接到主控制器中
            load_worker.run()
        self.settings = ServerSettings(processes = 0, timeout = timeout,
                        max_client = max_client)

        atexit.register(self.stop)

    def listen(self, port = 8080, host = ''):
        self.port = port
        self.host = host

    def run(self, processes = 0):
        try:
            self._run(processes)
            for p in self.run_pool:
                p.join()
        except KeyboardInterrupt:
            self.stop()

    def stop(self):
        for p in self.run_pool:
            p.terminate()

    def _run(self, processes = 0):
        processes = processes or cpu_count() 
        self.settings.processes = processes
        if self.autoreload and processes != 1:  # 自动重载不支持多进程模式，自动关闭自动重载
            self.autoreload = False
            print("autoreload not suport multi procesess")

        _server = self.__made_server()
        if processes == 1: # 如果process值为1表示在主进程中执行
            self.__print_run_msg()
            _server.serve_forever()
        else:
            self._multi_run(server = _server, processes = processes)

            ipc_server = IPCServer()
            ipc_server.serve_forever()

    def __print_run_msg(self, processes = None):
        mess = "listen: http://%s:%s" % (self.host or '0.0.0.0', self.port)
        if processes:
            mess += ' processes: %d' % processes

        print(mess)

    def _multi_run(self, server = None, processes = None):
        _server = server or self.__made_server()
        processes = processes or cpu_count() #  在此方法中，如未指定processes则是表示以cpu核数为限定
        for i in range(processes):
            p = Process(target = _server.serve_forever, args = (True, ))
            self.run_pool.append(p)

        for p in self.run_pool: # 启动所有进程
            p.start()

        self.__print_run_msg(processes)

    def __made_server(self):
        return StreamServer((self.host, self.port), self._callback, 
                 autoreload = self.autoreload, settings = self.settings)




#!/usr/bin/env python
#coding:utf-8
# Author        : tuxpy
# Email         : q8886888@qq.com.com
# Last modified : 2015-06-12 12:44:18
# Filename      : http.py
# Description   : 
from __future__ import unicode_literals
from gale.http import HTTPRequest as _HTTPRequest, HTTPHeaders
from time import time

class HTTPRequest(_HTTPRequest):
    def __init__(self, env):
        self.method = env['REQUEST_METHOD']
        self.uri = self.path = env['PATH_INFO']
        self.query = env.get('QUERY_STRING')
        if self.query:
            self.uri += '?' + self.query
        self.version = 'HTTP/1.1'
        self.version_num = self.__version_num()
        if env.get('HTTP_HOST'):
            self.host = env['HTTP_HOST']
        else:
            self.host = env['SERVER_NAME']

        self.host = self.host.split(':')[0]

        self.port = env.get('SERVER_PORT', 80)

        headers = HTTPHeaders(is_request = True)
        if env.get('CONTENT_TYPE'):
            dict.__setitem__(headers, 'Content-Type', env['CONTENT_TYPE'])

        if env.get('CONTENT_LENGTH'):
            content_length = env.get('CONTENT_LENGTH')
            dict.__setitem__(headers, 'Content-Length', content_length)
            self.body = env['wsgi.input'].read(int(content_length))
        else:
            self.body = ''

        self.remote_addr = env['REMOTE_ADDR']
        self._start_time = time()

        for key in env:
            if key.startswith("HTTP_"):
                dict.__setitem__(headers, key[5:].title().replace('_', '-'), env[key])

        self.headers = headers
        self.cookies = self.__get_cookies()
        self.files = {}
        self._body_arguments = {}

    @property
    def client_ip(self):
        return self.remote_addr





#!/usr/bin/env python
#coding:utf-8
# Author        : tuxpy
# Email         : q8886888@qq.com.com
# Last modified : 2015-06-12 12:29:59
# Filename      : stream.py
# Description   : 
from wsgiref import simple_server
from gale.autoreload import check_files
from functools import partial
from threading import Thread
from time import sleep
from gale.utils import set_close_exec

def wsgi_callback(cls, env, start_response):
    from .web import WSGIApplication
    wsgi_app = WSGIApplication()
    return wsgi_app(env, start_response, cls)

def monitor_files():
    while True:
        check_files()
        sleep(1)

class StreamServer(object):
    """为了兼容py3，py3的server用得是wsgi server"""
    def __init__(self, listen_add, callback, max_client = 1000, reuse_add = True, timeout = 15, autoreload = False):
        host, port = listen_add
        _server = simple_server.make_server(host, port, partial(wsgi_callback, callback))
        _server.timeout = timeout
        _server.request_queue_size = max_client
        _server.allow_reuse_address = reuse_add
        set_close_exec(_server.fileno())

        self._server = _server
        self.autoreload = autoreload

    def __autoreload(self):
        t = Thread(target = monitor_files)
        t.setDaemon(True)
        t.start()

    def serve_forever(self):
        if self.autoreload:
            self.__autoreload()
        self._server.serve_forever()




#!/usr/bin/env python
#coding:utf-8
# Author        : tuxpy
# Email         : q8886888@qq.com.com
# Last modified : 2016-02-18 21:18:52
# Filename      : running.py
# Description   : 
from __future__ import print_function




#!/usr/bin/env python
#coding:utf-8
# Author        : tuxpy
# Email         : q8886888@qq.com.com
# Last modified : 2015-09-25 16:09:31
# Filename      : restapi.py
# Description   : 
from __future__ import unicode_literals, print_function
import re
from gale import e
from gale.escape import to_unicode

__all__ = ['RestApi']

_ALL_METHOD = ('POST', 'GET', 'PUT', 'DELETE', 'HEAD')
re_resetapi_flag = re.compile(r'^rest\s*api:')

class DocParser(object):
    def __init__(self, raw_data = None):
        self.raw_data = raw_data

    def is_space_line(self, line):
        return not line.strip()

    def parse(self, raw_data = None):
        raw_data = raw_data or self.raw_data
        if not raw_data:
            return raw_data

        current_fork = tree = {}
        parent_line = None
        last_node = None

        for line in raw_data.split('\n'):
            if self.is_space_line(line):
                continue
            key, value = self.parse_line(line)

            if parent_line and self.entry_child_node(line, parent_line):
                current_fork[key] = value
            else:
                if last_node and (not current_fork):
                    tree[last_node] = self.parse_line(parent_line)[1]
                current_fork = tree.setdefault(key, {})
                last_node = key
                parent_line = line

        return tree

    def entry_child_node(self, line, last_line):
        return self.get_line_indent(line) > self.get_line_indent(last_line)

    def get_line_indent(self, line):
        return len(line) - len(line.strip())

    def parse_line(self, line):
        if ':' in line:
            return [item.strip() for item in line.split(':', 1)]
        else:
            raise e.DocParserError("':' must in line")

def parse_doc(raw_doc):
    if not raw_doc:
        return None
    doc_parser = DocParser(raw_doc)
    return doc_parser.parse()

class RestApi(object):
    def __init__(self, handlers):
        self.vhost_handlers = handlers

    def __get_all_handlers(self):
        """把所有的handlers以及相关的url列出来"""
        handlers = []
        for _host, _handlers in self.vhost_handlers:
            for re_url, _handler, kwargs in _handlers:
                handlers.append((re_url.pattern[1: -1],
                    _handler))

        return handlers

    def generate_restapi_list(self):
        all_handlers = self.__get_all_handlers()
        all_api_handlers = {}
        for url, handler in all_handlers:
            _api_handler_info = self.genearte_resetapi_info(handler)
            if not _api_handler_info: # 表示该handler不包含reset api信息
                continue

            module_name = handler.__module__
            _module = __import__(module_name, fromlist = [module_name])
            module_doc = getattr(_module, 'API_MODULE_DOC', None)
            if module_doc:
                module_name += '(%s)' % (to_unicode(module_doc), )

            all_api_handlers.setdefault(module_name,
                    []).append((url, handler, _api_handler_info))

        return all_api_handlers

    def genearte_resetapi_info(self, handler):
        """生成api处理类各方法的信息"""
        handler_api_doc = []
        for method in _ALL_METHOD:
            method_handler = getattr(handler, method, None)
            if not (method_handler and self.is_resetapi_method(method_handler)):
                continue
            api_method_doc = self.parse_method_doc(to_unicode(method_handler.__doc__))
            api_method_doc['method'] = method
            handler_api_doc.append(api_method_doc)

        return handler_api_doc

    def parse_method_doc(self, doc):
        return parse_doc(doc)

    def is_resetapi_method(self, method):
        """通过解析method的__doc__来得出这是不是个rest api"""
        doc = to_unicode((method.__doc__ or '').strip())
        if doc == '':
            return False
        return bool(re_resetapi_flag.match(doc))




#!/usr/bin/env python
#coding:utf8
# Author          : tuxpy
# Email           : q8886888@qq.com
# Last modified   : 2015-03-25 14:18:30
# Filename        : gale/e.py
# Description     : 

class HeaderFormatError(Exception):
    pass

class DocParserError(Exception):
    pass

class HTTPError(Exception):
    def __init__(self, status_code, status_mess=None):
        self.status_code = status_code
        self.status_mess = status_mess
        super(HTTPError, self).__init__()


class ErrorStatusCode(Exception):
    pass

class MissArgument(Exception):
    pass

class NotSupportHttpVersion(Exception):
    pass

class LoginHandlerNotExists(Exception):
    pass

class LocalPathNotExist(Exception):
    pass

class CookieError(Exception):
    pass

class CacheError(Exception):
    pass

class HasFinished(Exception):
    pass

class WebSocketError(Exception):
    pass

class IPCError(Exception):
    pass

class JSONError(Exception):
    pass

class NotSupportStrategy(Exception):
    pass

class WorkerOffline(Exception):
    pass


NotSupportMethod = HTTPError(405)




#!/usr/bin/env python
from __future__ import absolute_import, division, print_function, with_statement

import datetime
import linecache
import os.path
import posixpath
import re
import threading

from gale import escape
from gale.log import gen_log
from gale.utils import  exec_in, unicode_type

try:
    from cStringIO import StringIO  # py2
except ImportError:
    from io import StringIO  # py3

_DEFAULT_AUTOESCAPE = "xhtml_escape"
_UNSET = object()

class Template(object):
    """A compiled template.

    We compile into Python from the given template_string. You can generate
    the template from variables with generate().
    """
    # note that the constructor's signature is not extracted with
    # autodoc because _UNSET looks like garbage.  When changing
    # this signature update website/sphinx/template.rst too.
    def __init__(self, template_string, name="<string>", loader=None,
                 compress_whitespace=None, autoescape=_UNSET):
        self.name = name
        if compress_whitespace is None:
            compress_whitespace = name.endswith(".html") or \
                name.endswith(".js")
        if autoescape is not _UNSET:
            self.autoescape = autoescape
        elif loader:
            self.autoescape = loader.autoescape
        else:
            self.autoescape = _DEFAULT_AUTOESCAPE
        self.namespace = loader.namespace if loader else {}
        reader = _TemplateReader(name, escape.native_str(template_string))
        self.file = _File(self, _parse(reader, self))
        self.code = self._generate_python(loader, compress_whitespace)
        self.loader = loader
        try:
            # Under python2.5, the fake filename used here must match
            # the module name used in __name__ below.
            # The dont_inherit flag prevents template.py's future imports
            # from being applied to the generated code.
            self.compiled = compile(
                escape.to_unicode(self.code),
                "%s.generated.py" % self.name.replace('.', '_'),
                "exec", dont_inherit=True)
        except Exception:
            formatted_code = _format_code(self.code).rstrip()
            gen_log.error("%s code:\n%s", self.name, formatted_code)
            raise

    def generate(self, **kwargs):
        """Generate this template with the given arguments."""
        namespace = {
            "escape": escape.xhtml_escape,
            "xhtml_escape": escape.xhtml_escape,
            "datetime": datetime,
            "_tt_utf8": escape.utf8,  # for internal use
            "_tt_string_types": (unicode_type, bytes),
            # __name__ and __loader__ allow the traceback mechanism to find
            # the generated source code.
            "__name__": self.name.replace('.', '_'),
        }
        namespace.update(self.namespace)
        namespace.update(kwargs)
        exec_in(self.compiled, namespace)
        execute = namespace["_tt_execute"]
        # Clear the traceback module's cache of source data now that
        # we've generated a new template (mainly for this module's
        # unittests, where different tests reuse the same name).
        linecache.clearcache()
        return execute()

    def _generate_python(self, loader, compress_whitespace):
        buffer = StringIO()
        try:
            # named_blocks maps from names to _NamedBlock objects
            named_blocks = {}
            ancestors = self._get_ancestors(loader)
            ancestors.reverse()
            for ancestor in ancestors:
                ancestor.find_named_blocks(loader, named_blocks)
            writer = _CodeWriter(buffer, named_blocks, loader, ancestors[0].template,
                                 compress_whitespace)
            ancestors[0].generate(writer)
            return buffer.getvalue()
        finally:
            buffer.close()

    def _get_ancestors(self, loader):
        ancestors = [self.file]
        for chunk in self.file.body.chunks:
            if isinstance(chunk, _ExtendsBlock):
                if not loader:
                    raise ParseError("{% extends %} block found, but no "
                                     "template loader")
                template = loader.load(chunk.name, self.name)
                ancestors.extend(template._get_ancestors(loader))
        return ancestors


class BaseLoader(object):
    """Base class for template loaders.

    You must use a template loader to use template constructs like
    ``{% extends %}`` and ``{% include %}``. The loader caches all
    templates after they are loaded the first time.
    """
    def __init__(self, autoescape=_DEFAULT_AUTOESCAPE, namespace=None):
        """``autoescape`` must be either None or a string naming a function
        in the template namespace, such as "xhtml_escape".
        """
        self.autoescape = autoescape
        self.namespace = namespace or {}
        self.templates = {}
        # self.lock protects self.templates.  It's a reentrant lock
        # because templates may load other templates via `include` or
        # `extends`.  Note that thanks to the GIL this code would be safe
        # even without the lock, but could lead to wasted work as multiple
        # threads tried to compile the same template simultaneously.
        self.lock = threading.RLock()

    def reset(self):
        """Resets the cache of compiled templates."""
        with self.lock:
            self.templates = {}

    def resolve_path(self, name, parent_path=None):
        """Converts a possibly-relative path to absolute (used internally)."""
        raise NotImplementedError()

    def load(self, name, parent_path=None):
        """Loads a template."""
        name = self.resolve_path(name, parent_path=parent_path)
        with self.lock:
            if name not in self.templates:
                self.templates[name] = self._create_template(name)
            return self.templates[name]

    def _create_template(self, name):
        raise NotImplementedError()


class Loader(BaseLoader):
    """A template loader that loads from a single root directory.
    """
    def __init__(self, root_directory, **kwargs):
        super(Loader, self).__init__(**kwargs)
        self.root = os.path.abspath(root_directory)

    def resolve_path(self, name, parent_path=None):
        if parent_path and not parent_path.startswith("<") and \
            not parent_path.startswith("/") and \
                not name.startswith("/"):
            current_path = os.path.join(self.root, parent_path)
            file_dir = os.path.dirname(os.path.abspath(current_path))
            relative_path = os.path.abspath(os.path.join(file_dir, name))
            if relative_path.startswith(self.root):
                name = relative_path[len(self.root) + 1:]
        return name

    def _create_template(self, name):
        path = os.path.join(self.root, name)
        with open(path, "rb") as f:
            template = Template(f.read(), name=name, loader=self)
            return template

class StringLoader(BaseLoader):
    def __init__(self, chunk = '', **kwargs):
        super(StringLoader, self).__init__(**kwargs)
        self.chunk = chunk

    def resolve_path(self, *args, **kwargs):
        return None

    def load(self, string):
        return self._create_template(string)

    def _create_template(self, string):
        return Template(string, loader = self)

class DictLoader(BaseLoader):
    """A template loader that loads from a dictionary."""
    def __init__(self, dict, **kwargs):
        super(DictLoader, self).__init__(**kwargs)
        self.dict = dict

    def resolve_path(self, name, parent_path=None):
        if parent_path and not parent_path.startswith("<") and \
            not parent_path.startswith("/") and \
                not name.startswith("/"):
            file_dir = posixpath.dirname(parent_path)
            name = posixpath.normpath(posixpath.join(file_dir, name))
        return name

    def _create_template(self, name):
        return Template(self.dict[name], name=name, loader=self)


class _Node(object):
    def each_child(self):
        return ()

    def generate(self, writer):
        raise NotImplementedError()

    def find_named_blocks(self, loader, named_blocks):
        for child in self.each_child():
            child.find_named_blocks(loader, named_blocks)


class _File(_Node):
    def __init__(self, template, body):
        self.template = template
        self.body = body
        self.line = 0

    def generate(self, writer):
        writer.write_line("def _tt_execute():", self.line)
        with writer.indent():
            writer.write_line("_tt_buffer = []", self.line)
            writer.write_line("_tt_append = _tt_buffer.append", self.line)
            self.body.generate(writer)
            writer.write_line("return _tt_utf8('').join(_tt_buffer)", self.line)

    def each_child(self):
        return (self.body,)


class _ChunkList(_Node):
    def __init__(self, chunks):
        self.chunks = chunks

    def generate(self, writer):
        for chunk in self.chunks:
            chunk.generate(writer)

    def each_child(self):
        return self.chunks


class _NamedBlock(_Node):
    def __init__(self, name, body, template, line):
        self.name = name
        self.body = body
        self.template = template
        self.line = line

    def each_child(self):
        return (self.body,)

    def generate(self, writer):
        block = writer.named_blocks[self.name]
        with writer.include(block.template, self.line):
            block.body.generate(writer)

    def find_named_blocks(self, loader, named_blocks):
        named_blocks[self.name] = self
        _Node.find_named_blocks(self, loader, named_blocks)


class _ExtendsBlock(_Node):
    def __init__(self, name):
        self.name = name


class _IncludeBlock(_Node):
    def __init__(self, name, reader, line):
        self.name = name
        self.template_name = reader.name
        self.line = line

    def find_named_blocks(self, loader, named_blocks):
        included = loader.load(self.name, self.template_name)
        included.file.find_named_blocks(loader, named_blocks)

    def generate(self, writer):
        included = writer.loader.load(self.name, self.template_name)
        with writer.include(included, self.line):
            included.file.body.generate(writer)


class _ApplyBlock(_Node):
    def __init__(self, method, line, body=None):
        self.method = method
        self.line = line
        self.body = body

    def each_child(self):
        return (self.body,)

    def generate(self, writer):
        method_name = "_tt_apply%d" % writer.apply_counter
        writer.apply_counter += 1
        writer.write_line("def %s():" % method_name, self.line)
        with writer.indent():
            writer.write_line("_tt_buffer = []", self.line)
            writer.write_line("_tt_append = _tt_buffer.append", self.line)
            self.body.generate(writer)
            writer.write_line("return _tt_utf8('').join(_tt_buffer)", self.line)
        writer.write_line("_tt_append(_tt_utf8(%s(%s())))" % (
            self.method, method_name), self.line)


class _ControlBlock(_Node):
    def __init__(self, statement, line, body=None):
        self.statement = statement
        self.line = line
        self.body = body

    def each_child(self):
        return (self.body,)

    def generate(self, writer):
        writer.write_line("%s:" % self.statement, self.line)
        with writer.indent():
            self.body.generate(writer)
            # Just in case the body was empty
            writer.write_line("pass", self.line)


class _IntermediateControlBlock(_Node):
    def __init__(self, statement, line):
        self.statement = statement
        self.line = line

    def generate(self, writer):
        # In case the previous block was empty
        writer.write_line("pass", self.line)
        writer.write_line("%s:" % self.statement, self.line, writer.indent_size() - 1)


class _Statement(_Node):
    def __init__(self, statement, line):
        self.statement = statement
        self.line = line

    def generate(self, writer):
        writer.write_line(self.statement, self.line)


class _Expression(_Node):
    def __init__(self, expression, line, raw=False):
        self.expression = expression
        self.line = line
        self.raw = raw

    def generate(self, writer):
        writer.write_line("_tt_tmp = %s" % self.expression, self.line)
        writer.write_line("if isinstance(_tt_tmp, _tt_string_types):"
                          " _tt_tmp = _tt_utf8(_tt_tmp)", self.line)
        writer.write_line("else: _tt_tmp = _tt_utf8(str(_tt_tmp))", self.line)
        if not self.raw and writer.current_template.autoescape is not None:
            # In python3 functions like xhtml_escape return unicode,
            # so we have to convert to utf8 again.
            writer.write_line("_tt_tmp = _tt_utf8(%s(_tt_tmp))" %
                              writer.current_template.autoescape, self.line)
        writer.write_line("_tt_append(_tt_tmp)", self.line)


class _Module(_Expression):
    def __init__(self, expression, line):
        super(_Module, self).__init__("_tt_modules." + expression, line,
                                      raw=True)


class _Text(_Node):
    def __init__(self, value, line):
        self.value = value
        self.line = line

    def generate(self, writer):
        value = self.value

        # Compress lots of white space to a single character. If the whitespace
        # breaks a line, have it continue to break a line, but just with a
        # single \n character
        if writer.compress_whitespace and "<pre>" not in value:
            value = re.sub(r"([\t ]+)", " ", value)
            value = re.sub(r"(\s*\n\s*)", "\n", value)

        if value:
            writer.write_line('_tt_append(%r)' % escape.utf8(value), self.line)


class ParseError(Exception):
    """Raised for template syntax errors."""
    pass


class _CodeWriter(object):
    def __init__(self, file, named_blocks, loader, current_template,
                 compress_whitespace):
        self.file = file
        self.named_blocks = named_blocks
        self.loader = loader
        self.current_template = current_template
        self.compress_whitespace = compress_whitespace
        self.apply_counter = 0
        self.include_stack = []
        self._indent = 0

    def indent_size(self):
        return self._indent

    def indent(self):
        class Indenter(object):
            def __enter__(_):
                self._indent += 1
                return self

            def __exit__(_, *args):
                assert self._indent > 0
                self._indent -= 1

        return Indenter()

    def include(self, template, line):
        self.include_stack.append((self.current_template, line))
        self.current_template = template

        class IncludeTemplate(object):
            def __enter__(_):
                return self

            def __exit__(_, *args):
                self.current_template = self.include_stack.pop()[0]

        return IncludeTemplate()

    def write_line(self, line, line_number, indent=None):
        if indent is None:
            indent = self._indent
        line_comment = '  # %s:%d' % (self.current_template.name, line_number)
        if self.include_stack:
            ancestors = ["%s:%d" % (tmpl.name, lineno)
                         for (tmpl, lineno) in self.include_stack]
            line_comment += ' (via %s)' % ', '.join(reversed(ancestors))
        print("    " * indent + escape.utf8(line) + escape.utf8(line_comment), file=self.file)


class _TemplateReader(object):
    def __init__(self, name, text):
        self.name = name
        self.text = text
        self.line = 1
        self.pos = 0

    def find(self, needle, start=0, end=None):
        assert start >= 0, start
        pos = self.pos
        start += pos
        if end is None:
            index = self.text.find(needle, start)
        else:
            end += pos
            assert end >= start
            index = self.text.find(needle, start, end)
        if index != -1:
            index -= pos
        return index

    def consume(self, count=None):
        if count is None:
            count = len(self.text) - self.pos
        newpos = self.pos + count
        self.line += self.text.count("\n", self.pos, newpos)
        s = self.text[self.pos:newpos]
        self.pos = newpos
        return s

    def remaining(self):
        return len(self.text) - self.pos

    def __len__(self):
        return self.remaining()

    def __getitem__(self, key):
        if type(key) is slice:
            size = len(self)
            start, stop, step = key.indices(size)
            if start is None:
                start = self.pos
            else:
                start += self.pos
            if stop is not None:
                stop += self.pos
            return self.text[slice(start, stop, step)]
        elif key < 0:
            return self.text[key]
        else:
            return self.text[self.pos + key]

    def __str__(self):
        return self.text[self.pos:]


def _format_code(code):
    lines = code.splitlines()
    format = "%%%dd  %%s\n" % len(repr(len(lines) + 1))
    return "".join([format % (i + 1, line) for (i, line) in enumerate(lines)])


def _parse(reader, template, in_block=None, in_loop=None):
    body = _ChunkList([])
    while True:
        # Find next template directive
        curly = 0
        while True:
            curly = reader.find("{", curly)
            if curly == -1 or curly + 1 == reader.remaining():
                # EOF
                if in_block:
                    raise ParseError("Missing {%% end %%} block for %s" %
                                     in_block)
                body.chunks.append(_Text(reader.consume(), reader.line))
                return body
            # If the first curly brace is not the start of a special token,
            # start searching from the character after it
            if reader[curly + 1] not in ("{", "%", "#"):
                curly += 1
                continue
            # When there are more than 2 curlies in a row, use the
            # innermost ones.  This is useful when generating languages
            # like latex where curlies are also meaningful
            if (curly + 2 < reader.remaining() and
                    reader[curly + 1] == '{' and reader[curly + 2] == '{'):
                curly += 1
                continue
            break

        # Append any text before the special token
        if curly > 0:
            cons = reader.consume(curly)
            body.chunks.append(_Text(cons, reader.line))

        start_brace = reader.consume(2)
        line = reader.line

        # Template directives may be escaped as "{{!" or "{%!".
        # In this case output the braces and consume the "!".
        # This is especially useful in conjunction with jquery templates,
        # which also use double braces.
        if reader.remaining() and reader[0] == "!":
            reader.consume(1)
            body.chunks.append(_Text(start_brace, line))
            continue

        # Comment
        if start_brace == "{#":
            end = reader.find("#}")
            if end == -1:
                raise ParseError("Missing end expression #} on line %d" % line)
            contents = reader.consume(end).strip()
            reader.consume(2)
            continue

        # Expression
        if start_brace == "{{":
            end = reader.find("}}")
            if end == -1:
                raise ParseError("Missing end expression }} on line %d" % line)
            contents = reader.consume(end).strip()
            reader.consume(2)
            if not contents:
                raise ParseError("Empty expression on line %d" % line)
            body.chunks.append(_Expression(contents, line))
            continue

        # Block
        assert start_brace == "{%", start_brace
        end = reader.find("%}")
        if end == -1:
            raise ParseError("Missing end block %%} on line %d" % line)
        contents = reader.consume(end).strip()
        reader.consume(2)
        if not contents:
            raise ParseError("Empty block tag ({%% %%}) on line %d" % line)

        operator, space, suffix = contents.partition(" ")
        suffix = suffix.strip()

        # Intermediate ("else", "elif", etc) blocks
        intermediate_blocks = {
            "else": set(["if", "for", "while", "try"]),
            "elif": set(["if"]),
            "except": set(["try"]),
            "finally": set(["try"]),
        }
        allowed_parents = intermediate_blocks.get(operator)
        if allowed_parents is not None:
            if not in_block:
                raise ParseError("%s outside %s block" %
                                 (operator, allowed_parents))
            if in_block not in allowed_parents:
                raise ParseError("%s block cannot be attached to %s block" % (operator, in_block))
            body.chunks.append(_IntermediateControlBlock(contents, line))
            continue

        # End tag
        elif operator == "end":
            if not in_block:
                raise ParseError("Extra {%% end %%} block on line %d" % line)
            return body

        elif operator in ("extends", "include", "set", "import", "from",
                          "comment", "autoescape", "raw", "module"):
            if operator == "comment":
                continue
            if operator == "extends":
                suffix = suffix.strip('"').strip("'")
                if not suffix:
                    raise ParseError("extends missing file path on line %d" % line)
                block = _ExtendsBlock(suffix)
            elif operator in ("import", "from"):
                if not suffix:
                    raise ParseError("import missing statement on line %d" % line)
                block = _Statement(contents, line)
            elif operator == "include":
                suffix = suffix.strip('"').strip("'")
                if not suffix:
                    raise ParseError("include missing file path on line %d" % line)
                block = _IncludeBlock(suffix, reader, line)
            elif operator == "set":
                if not suffix:
                    raise ParseError("set missing statement on line %d" % line)
                block = _Statement(suffix, line)
            elif operator == "autoescape":
                fn = suffix.strip()
                if fn == "None":
                    fn = None
                template.autoescape = fn
                continue
            elif operator == "raw":
                block = _Expression(suffix, line, raw=True)
            elif operator == "module":
                block = _Module(suffix, line)
            body.chunks.append(block)
            continue

        elif operator in ("apply", "block", "try", "if", "for", "while"):
            # parse inner body recursively
            if operator in ("for", "while"):
                block_body = _parse(reader, template, operator, operator)
            elif operator == "apply":
                # apply creates a nested function so syntactically it's not
                # in the loop.
                block_body = _parse(reader, template, operator, None)
            else:
                block_body = _parse(reader, template, operator, in_loop)

            if operator == "apply":
                if not suffix:
                    raise ParseError("apply missing method name on line %d" % line)
                block = _ApplyBlock(suffix, line, block_body)
            elif operator == "block":
                if not suffix:
                    raise ParseError("block missing name on line %d" % line)
                block = _NamedBlock(suffix, block_body, template, line)
            else:
                block = _ControlBlock(contents, line, block_body)
            body.chunks.append(block)
            continue

        elif operator in ("break", "continue"):
            if not in_loop:
                raise ParseError("%s outside %s block" % (operator, set(["for", "while"])))
            body.chunks.append(_Statement(contents, line))
            continue

        else:
            raise ParseError("unknown operator: %r" % operator)



#!/usr/bin/env python
#coding:utf-8
# Author        : tuxpy
# Email         : q8886888@qq.com.com
# Last modified : 2015-06-11 15:37:12
# Filename      : wsgi.py
# Description   : 
from __future__ import print_function
from .http import HTTPRequest
from gale.escape import native_str
from gale.web import Application

class WSGIApplication(Application):
    def __call__(self, env, start_response, app_instance = None):
        app_instance = app_instance or self
        http_request = HTTPRequest(env)
        http_request._parse_body()
        from gale import web
        handler = Application.__call__(app_instance, http_request, is_wsgi = True)

        if hasattr(handler, '_new_cookie'):
            for cookie in handler._new_cookie.values():
                handler.add_header('Set-Cookie', cookie.OutputString())


        status = "%s %s" % (handler._status_code, handler._status_mess)


        _body = b''.join(handler._push_buffer)

        if http_request.method == 'HEAD': # 如果是HEAD请求的话则不返回主体内容
            _body = b''

        if handler._status_code != 304: 
            handler.set_header('Content-Length', len(_body))

        headers = [(native_str(_k), str(native_str(_v))) for _k, _v in handler._headers.get_headers_items()] # 在python2.7 中，不允许headers是unicode
        write = start_response(native_str(status), headers) 

        write(_body)
        handler.log_print()
        handler.on_finish()
        return []




#!/usr/bin/env python
#coding:utf-8
# Author        : tuxpy
# Email         : q8886888@qq.com.com
# Last modified : 2015-07-05 13:38:30
# Filename      : autoreload.py
# Description   : 
import sys
import os
import signal
from gale import utils

modify_times = {}

def __check_modify(modify_times, path):
    try:
        st_mtime = os.stat(path).st_mtime
    except OSError:
        return

    if path not in modify_times: # 表示是新的
        modify_times[path] = st_mtime
        return 

    if modify_times[path] != st_mtime: # 表示已经有更新过了
        __reload_files()

def __reload_files():
    utils.stop_share_dict()
    os.execv(sys.executable, [sys.executable] + sys.argv)

def check_files():
    """检查文件的修改时间，检查对象是sys.path中的路径"""
    for module in sys.modules.values():
        path = getattr(module, '__file__', None)
        if not path:
            continue
        suffix = path.split('.')[-1]
        if suffix in ('pyc', 'pyo'):
            path = path[:-1]

        __check_modify(modify_times, path)




#!/usr/bin/env python
#coding:utf-8
# Author        : tuxpy
# Email         : q8886888@qq.com.com
# Last modified : 2015-09-15 08:31:23
# Filename      : ipc.py
# Description   : 
from __future__ import print_function
from gale.utils import set_close_exec, ObjectDict, single_pattern
from gale.e import IPCError
from gevent import socket
import gevent
import json
from multiprocessing import Process
import os
import tempfile
import time
import struct
from functools import partial
import signal

__all__ = ['IPCServer','IPCDict']

ALL_OPERA = {'set': False, 'get': True, 'items': True, 'has_key': True,
        'pop': True, 'popitem': True, 'update': False, 'values': True,
        'setdefault': True, 'keys': True, 'clear': False, 'del': False, 
        'incr': True}

class FRAME():
    DATA_LENGTH = 8 # 数据的位数

def genearte_sock_path(is_sub = False):
    return '/tmp/gale.sock'

class _Memory(dict):
    def _set(self, name, value):
        self[name] = value

    def _del(self, name):
        try:
            del self[name]
        except KeyError:
            pass

    def _incr(self, name, increment = 1):
        if not isinstance(increment, int):
            raise TypeError('increment type must be int')
        current = self.setdefault(name, 0)
        self[name] = current + increment

        return self[name]

@single_pattern
class IPCMemory(object):
    _memory_block = {}

    def __getitem__(self, name):
        return self._memory_block.setdefault(name, _Memory())

@single_pattern
class IPCServer(object):
    def __init__(self):
        self._socket = socket.socket(socket.AF_UNIX, socket.SOCK_STREAM)
        set_close_exec(self._socket.fileno())
        sock_path = genearte_sock_path()
        try:
            os.remove(sock_path)
        except OSError as e:
            if e.errno != 2: # 2表示no such file or direrctory
                raise e
            
        self._socket.bind(genearte_sock_path())
        self._socket.listen(50)

    def serve_forever(self):
        Process(target = self._serve_forever).start()

    def _serve_forever(self):
        signal.signal(signal.SIGINT, self.close)
        while not self._socket.closed:
            try:
                conn, addr = self._socket.accept()
            except socket.error:
                self.close()
                break
            connection = IPCConnection(conn)
            gevent.spawn(connection.start_work)

    def close(self, *args):
        if self._socket.closed:
            return

        os.remove(self._socket.getsockname())
        self._socket.close()

class Connection(object):
    def __init__(self, _socket):
        self._socket = _socket

    def recv_parse_frame(self):
        frame = ObjectDict()
        self.on_frame_header(frame)
        if not frame:
            self.close()
            return 

        self.on_frame_data(frame)

        return frame

    def close(self):
        self._socket.close()

    def on_frame_header(self, frame):
        _header = ObjectDict()
        header = self._socket.recv(FRAME.DATA_LENGTH) # 先获取ipc数据库
        if not header:
            return

        data_length = struct.unpack(b'!Q', header)[0]
        _header.data_length = data_length

        frame.update(_header)

    def on_frame_data(self, frame):
        data_length = frame.data_length
        data = self._recv_data(data_length)
        frame.data = data

    def _recv_data(self, data_length):
        assert data_length > 0
        chunk = []
        while True:
            _data = self._socket.recv(data_length)
            _len = len(_data)
            assert _len <= data_length

            chunk.append(_data)
            if len(_data) == data_length:
                break
            data_length -= _len

        return b''.join(chunk)

    def _send_data(self, data):
        """这里的data是已经处理好了的"""
        frame = b''
        data_length = len(data)
        frame += struct.pack(b'!Q', data_length)
        frame += data
        self._socket.sendall(frame)

class IPCConnection(Connection):
    def start_work(self):
        while True:
            frame = self.recv_parse_frame()
            if not frame:
                break
            self.exec_ipc(json.loads(frame.data))


    def exec_ipc(self, data):
        ipc_memory = IPCMemory()
        name, command, args, kwargs = data
        need_return = ALL_OPERA[command]

        if command not in dir({}):
            command = '_' + command

        exec_func = getattr(ipc_memory[name], command)
        result = exec_func(*args, **kwargs)

        if not need_return: # 如果不需要返回值，则就此结束了
            return
        self._send_data(json.dumps(result))

class IPCClient(Connection):
    def __init__(self, name):
        _socket = socket.socket(socket.AF_UNIX, socket.SOCK_STREAM)
        sock_path = genearte_sock_path(True)
        _socket.connect(sock_path)
        self.name = name
        Connection.__init__(self, _socket)

    def _exec_ipc(self, command, *args, **kwargs):
        need_return =  ALL_OPERA[command]
        frame_data = self.generate_frame_data(command, *args, **kwargs)
        self._send_data(frame_data)
        if not need_return:
            return

        return self.__get_ipc_return()

    def __get_ipc_return(self):
        frame = self.recv_parse_frame()
        if not frame:
            return
        
        data = frame.data
        return json.loads(data)

    def generate_frame_data(self, command, *args, **kwargs):
        return json.dumps([self.name, command, args, kwargs])

class IPCDict(IPCClient): 
    def __getattr__(self, name):
        if name.startswith('_'):
            return self.__dict__[name]

        if name not in ALL_OPERA:
            name = 'get'

        return partial(self._exec_ipc, name)

    def __getitem__(self, name):
        return self.get(name)

    def __setitem__(self, name, value):
        self.set(name, value)


    def __delitem__(self, name):
        return self._exec_ipc('del', name)




#coding:utf8
# Author          : tuxpy
# Email           : q8886888@qq.com
# Last modified   : 2015-03-26 13:14:11
# Filename        : gale/utils.py
# Description     : 
from __future__ import unicode_literals
try: # py2
    from urlparse import urlsplit 
    from urllib import unquote_plus
    from urllib import quote_plus
except ImportError: # py3
    from urllib.parse import urlsplit # py3
    from urllib.parse import unquote_plus
    from urllib.parse import quote_plus

import email.utils
import time
import urllib
from gale import escape
from gale.config import CRLF
import mimetypes
import uuid
import fcntl
import gevent
from gevent import (Greenlet, socket)
from functools import wraps
import sys

def set_close_exec(fd):
    flags = fcntl.fcntl(fd, fcntl.F_GETFD)
    fcntl.fcntl(fd, fcntl.F_SETFD, flags|fcntl.FD_CLOEXEC)
    # 设置close exec标志，这样在reload时会关闭socket

def get_gale_socket(raw_socket = None):
    _socket = raw_socket or socket.socket(socket.AF_INET,
            socket.SOCK_STREAM)
    _socket.setsockopt(socket.SOL_SOCKET,
            socket.SO_REUSEADDR, 1)
    _socket.setsockopt(socket.IPPROTO_TCP,
            socket.TCP_NODELAY, 1)

    set_close_exec(_socket.fileno())

    return _socket

def parse_request_range(_range):
    if not _range:
        return 0, 0
    if 'bytes=' not in _range:
        return 0, 0
    bytes_range = _range.split('bytes=')[-1].strip()
    start, stop = bytes_range.split('-')
    start = start and int(start) or 0
    stop = stop and int(stop) or 0

    return start, stop

def urldecode(params_url):
    if not params_url: # 如果没有东西的话，就返回{}
        return {}

    params_url = escape.param_decode(params_url)

    _d = {} # 存的是请求参数的字典形式，值是参数值列表
    for _name, _value in map(lambda x: x.split('=', 1), 
            filter(lambda k_v: '=' in k_v, params_url.split('&'))): # filter 是为了把不带有=号的参数去掉
        # 对用户提交的url参数和body进行解码成unicode

        _d.setdefault(_name, []).append(urlunquote(_value))
    return _d

def urlunquote(param):
    if param == None:
        return param
    param = unquote_plus(escape.native_str(param))
    return escape.param_decode(param)

def urlquote(param):
    return quote_plus(escape.utf8(param))

code_mess_map = {
            100: 'Continue',
            101: 'Switching Protocols',
            200: 'OK',
            201: 'Created',
            202: 'Accepted',
            203: 'Non-Authoritative Information',
            204: 'No Content',
            205: 'Reset Content',
            206: 'Partial Content',
            300: 'Multiple Choices',
            301: 'Moved Permanently',
            302: 'Found',
            303: 'See Other',
            304: 'Not Modified',
            305: 'Use Proxy',
            307: 'Temporary Redirect',
            400: 'Bad Request',
            401: 'Unauthorized',
            402: 'Payment Required',
            403: 'Forbidden',
            404: 'Not Found',
            405: 'Method Not Allowed',
            406: 'Not Acceptable',
            407: 'Proxy Authentication Required',
            408: 'Request Timeout',
            409: 'Conflict',
            410: 'Gone',
            411: 'Length Required',
            412: 'Precondition Failed',
            413: 'Request Entity Too Large',
            414: 'Request-URI Too Long',
            415: 'Unsupported Media Type',
            416: 'Requested Range Not Satisfiable',
            417: 'Expectation Failed',
            500: 'Internal Server Error',
            501: 'Not Implemented',
            502: 'Bad Gateway',
            503: 'Service Unavailable',
            504: 'Gateway Timeout',
            505: 'HTTP Version Not Supported'
            }

def format_timestamp(ts = None):
    if not ts:
        ts = time.time()
    return email.utils.formatdate(ts, usegmt = True)


def get_mime_type(file_path):
    return mimetypes.guess_type(file_path)[0] or 'application/octet-stream'

def made_uuid():
    return uuid.uuid4().hex
    
from multiprocessing import Manager
__mgr = Manager()
def ShareDict(*args, **kwargs):
    return __mgr.dict(*args, **kwargs)

def stop_share_dict():
    __mgr.shutdown()

from gale.py_ver import is_py3
unicode_type = is_py3 and str or unicode

def is_string(s):
    if is_py3:
        return isinstance(s, str)
    else:
        return isinstance(s, (str, unicode))

if is_py3:
    exec("""
def raise_exc_info(exc_info):
    raise exc_info[1].with_traceback(exc_info[2])

def exec_in(code, glob, loc=None):
    if isinstance(code, str):
        code = compile(code, '<string>', 'exec', dont_inherit=True)
    exec(code, glob, loc)
""")
else:
    exec("""
def raise_exc_info(exc_info):
    raise exc_info[0], exc_info[1], exc_info[2]

def exec_in(code, glob, loc=None):
    if isinstance(code, basestring):
        # exec(string) inherits the caller's future imports; compile
        # the string first to prevent that.
        code = compile(code, '<string>', 'exec', dont_inherit=True)
    exec code in glob, loc
""")


class ObjectDict(dict):
    def __setattr__(self, key, value):
        self[key] = value

    def __getattr__(self, key):
        return self[key]


def single_pattern(obj):
    @wraps(obj)
    def wrap(*args, **kwargs):
        if hasattr(obj, '_instance'):
            return obj._instance

        _instance = obj.__new__(obj, *args, **kwargs)
        obj.__init__(_instance, *args, **kwargs)
        obj._instance = _instance
        return _instance

    return wrap





#!/usr/bin/env python
#coding:utf8
# Author          : tuxpy
# Email           : q8886888@qq.com
# Last modified   : 2015-03-25 13:19:19
# Filename        : __init__.py
# Description     : 
from __future__ import print_function

__ALL__ = ['version']
version = '4.7.11'


def run_env_check():
    import platform
    if platform.python_version() < '2.6':
        raise RuntimeError('python version must be >= 2.6')

    if platform.system() not in ('Linux', 'Darwin'):
        raise RuntimeError('gale not supported %s' %(platform.system()))

run_env_check()




#!/usr/bin/env python
#coding:utf-8
# Author        : tuxpy
# Email         : q8886888@qq.com.com
# Last modified : 2015-08-25 19:59:30
# Filename      : coroutine.py
# Description   : 
import gevent
from gevent import Greenlet, monkey
from functools import wraps

class Coroutine(Greenlet):
    def __init__(self, func, args = None, kwargs = None, callback = None):
        Greenlet.__init__(self)
        self._args = args or ()
        self._kwargs = kwargs or {}
        self._callback = callback
        self._func = func

    def _run(self):
        error = None
        result = None
        try:
            result = self._func(*self._args, **self._kwargs)
        except Exception as e:
            error = e

        if self._callback:
            self._callback(result, error)


def coroutine(func):
    @wraps(func)
    def wrap(*args, **kwargs):
        monkey.patch_all()
        callback = kwargs.pop('callback', None)
        _coroutine = Coroutine(func, args, kwargs,
                callback = callback)
        _coroutine.start()

    return wrap



#!/usr/bin/env python
#coding:utf-8
# Author        : tuxpy
# Email         : q8886888@qq.com.com
# Last modified : 2015-04-19 12:57:50
# Filename      : stream.py
# Description   : 
from __future__ import print_function
import gevent
from gale.httpconnection import HTTPConnection 
from gale.autoreload import check_files
from gale import utils
import sys
import time
import os
from gevent import sleep
import signal

def monitor_files():
    while True:
        check_files()
        sleep(0.2)

class StreamServer(object):
    def __init__(self, listen_add, callback, settings, reuse_add = True, autoreload = False):
        self._socket = utils.get_gale_socket()


        self._socket.bind(listen_add)
        self._socket.listen(settings.max_client)
        self._callback = callback
        self.timeout = settings.timeout
        self.autoreload = autoreload

        self.settings = settings

    def __autoreload(self):
        gevent.spawn(monitor_files)

    def serve_forever(self, in_sub = False):
        if in_sub: # 只在子进程中才忽略掉中断信号
            signal.signal(signal.SIGINT, signal.SIG_IGN)

        if self.autoreload:
            self.__autoreload()

        while True:
            s, addr = self._socket.accept()
            s = utils.get_gale_socket(s)
            try:
                s.settimeout(self.timeout) # 设置客户端的超时时间
                connection = HTTPConnection(s)
                gevent.spawn(connection.get_request, self._callback,
                                server_settings = self.settings).link_exception(connection.gevent_exception) # 当成功生成一个request时，就会把request传入callback中去，一般是Application
            except Exception as e:
                s.close()

    def raise_error(self, g):
        g.kill()




#!/usr/bin/env python
#coding:utf-8
# Author        : tuxpy
# Email         : q8886888@qq.com.com
# Last modified : 2016-02-17 14:19:43
# Filename      : files.py
# Description   : 
from __future__ import unicode_literals, print_function
import sys
import os
from gale import web

def get_share_path():
    try:
        share_path = sys.argv[1]
    except IndexError:
        share_path = os.getcwd()

    return os.path.abspath(share_path)

def share():
    web.router(r'/(.*)', handler = web.FileHandler,
            kwargs = {
                'root'      :       get_share_path(),
                })
            
    web.app_run(port = 8000, processes = 1)

if __name__ == "__main__":
    share()





#coding:utf-8
# Author        : tuxpy
# Email         : q8886888@qq.com.com
# Last modified : 2015-03-29 18:47:12
# Filename      : escape.py
# Description   : 
from __future__ import unicode_literals
from gale.py_ver import is_py3
from json import dumps
import re

_XHTML_ESCAPE_RE = re.compile('[&<>"\']')
_XHTML_ESCAPE_DICT = {'&': '&amp;', '<': '&lt;', '>': '&gt;', '"': '&quot;',                      '\'': '&#39;'}

long_type = is_py3 and int or long
unicode_type = is_py3 and str or unicode

def utf8(param):
    if is_py3:
        if isinstance(param, str):
            return param.encode()
    else:
        if isinstance(param, unicode):
            return param.encode('utf-8')

    return param

def param_decode(param):
    if is_py3:
        if isinstance(param, unicode_type):
            return param
        elif isinstance(param, bytes):
            return param.decode('utf-8')
        else:
            return param

    if isinstance(param, str):
        return param.decode('utf-8')

    return param


def to_unicode(param):
    param = param_decode(param)
    return param

def param_encode(param):
    if is_py3:
        return param
    if isinstance(param, unicode):
        return param.encode('utf-8')

if is_py3:
    native_str = to_unicode
else:
    native_str = utf8


def xhtml_escape(value):
    return _XHTML_ESCAPE_RE.sub(lambda match: _XHTML_ESCAPE_DICT[match.group(0)],
            to_unicode(value))




#!/usr/bin/env python
#coding:utf-8
# Author        : tuxpy
# Email         : q8886888@qq.com.com
# Last modified : 2015-06-12 09:46:47
# Filename      : httpconnection.py
# Description   : 
from gale.config import CRLF
import traceback
from gale.http import HTTPHeaders, HTTPRequest
from gale.iosocket import IOSocket
from gevent import socket

class HTTPConnection(IOSocket):
    def parse_request_headers(self):
        """从原始数据中提取头信息"""
        _first_line, _headers = (self._headers.split(CRLF, 1) +  [''])[:2]
        self._headers = ''
        return _first_line, _headers

    def read_body(self, max_length):
        if max_length == 0: # 如果请求headers中没有指定 Content-Type或者为0,表示没有请求主体
            return b''

        body = self._buff # _buff里的内容是在读取 headers时多读取的数据
        while len(body) < max_length:
            _body_data = self._socket.recv(self.max_buff)
            if not _body_data:
                break
            body += _body_data

        self._buff = b''
        return body

    def read_headers(self):
        eof = -1
        _headers = b''
        while True:
            try:
                _data = self._socket.recv(self.max_buff)
            except socket.error:
                _data = None
            if not _data:
                break
            self._buff += _data
            eof = self._buff.find(CRLF * 2)
            if eof != -1:
                _headers, self._buff = self._buff[:eof], self._buff[eof + len(CRLF * 2):]
                break

        return _headers.strip()

    def get_request(self, callback, **kwargs):
        """获取用户的请求信息，并可以成功取得一个http请求时，生成一个HTTPRequest, 并激活回调函数(application的__call__)"""
        while True and self.closed == False:
            try:
                _headers = self.read_headers()
            except socket.timeout: # socket超时的异常不报
                _headers = None

            if not _headers:
                self.close()
                break

            self._headers = _headers
            request = get_request(self)
            kwargs['is_wsgi'] = False
            callback(request, **kwargs)

    def send_headers(self, headers_string):
        """headers_string是已经被处理过了的头信息，直接写入就行"""
        self.send_string(headers_string)

    def send_body(self, body_string):
        """同send_headers"""
        self.send_string(body_string)

    def send_finish_tag(self):
        self.send_string(CRLF)

    def remote_ip(self):
        return self._socket.getpeername()[0]

    def remote_port(self):
        return self._socket.getpeername()[1]

def get_request(connection, real_ip=True):
    """在刚连接时，获取用户的http请求信息, socket是客户端的socket"""
    if connection.is_close(): # 如果连接出错而被关闭，则返回 False，立刻结束本此请求
        return False
    try:
        _first_line, _headers = connection.parse_request_headers() # 把收到的信息分析出来
        headers = HTTPHeaders(_headers) # 这是http headers信息，类型是dict
        method , uri, version = map(lambda s: s.strip(), _first_line.split())
        _body = connection.read_body(int(headers.get('Content-Length', 0))) # 防止有body数据没有被读完，根据header中 的Content-Length再去读一下, 直到长度达到指定值
        http_request = HTTPRequest(method = method, uri = uri, version = version,
            headers = headers, body = _body, connection = connection, real_ip = real_ip)
        http_request._parse_body()
        return http_request

    except Exception as e:
        connection.close()
        traceback.print_exc()




#!/usr/bin/env python
#coding:utf-8
# Author        : tuxpy
# Email         : q8886888@qq.com.com
# Last modified : 2015-09-08 14:04:23
# Filename      : websocket.py
# Description   : 
from __future__ import unicode_literals, print_function
import base64
import struct
import hashlib
from gale.escape import utf8
from gale.utils import urlsplit, ObjectDict, is_string
from gale.web import RequestHandler, async, HTTPError
from gale.e import WebSocketError

WS = '258EAFA5-E914-47DA-95CA-C5AB0DC85B11'

def generate_response_key(websocket_key):
    return base64.encodestring(hashlib.sha1(websocket_key + WS).digest()).strip()


class WebSocketConnection(object):
    def __init__(self, websocket_version, websocket_key, handler):
        self.websocket_version = websocket_version
        self.websocket_key = websocket_key
        self.handler = handler
        self.stream = handler.request.connection
        self.stream.set_timeout(100)
        self.closed = False
        self.close_status = None
        self.close_reason = None

    def accept(self):
        response = [
                'HTTP/1.1 101 Switching Protocols',
                'Upgrade: websocket',
                'Connection: Upgrade',
                'Sec-WebSocket-Accept: %s' % (generate_response_key(self.websocket_key)),
                ]
        response = utf8('\r\n'.join(response) + '\r\n' * 2)
        self.stream.write(response)
        self.handler.on_open()
        while self.closed == False:
            frame_data = self.recv_frame_data()
            if frame_data.opcode == 0x1: # 接受text
                self.handler.on_message(frame_data)
            elif frame_data.opcode == 0x8: # 关闭
                _data = frame_data.data
                if len(_data) >= 2:
                    self.close_status = struct.unpack(b'!H', _data[:2])[0]
                if len(_data) > 2:
                    self.close_reason = _data[2:]
                self.close()

            elif frame_data.opcode == 0x9: # on ping
                self.handler.on_ping(frame_data)

    def close(self):
        if self.closed:
            return
        self.stream.close()
        self.handler.on_close(self.close_status, self.close_reason)
        self.closed = True

    def recv_frame_data(self):
        frame = ObjectDict()
        _header_data = self.recv_bytes(2)
        if not _header_data:
            return None
        self._on_frame_header(_header_data, frame)


        if frame.payload_len < 126:
            pass
        elif frame.payload_len == 126:
            frame.payload_len = struct.unpack(b'!H', self.recv_bytes(2))[0]
            # frame.payload_len = sum([ _d & (0xFF << (base * 8)) for base in range(2)])
        elif frame.payload_len == 127:
            frame.payload_len  = struct.unpack(b'!Q', self.recv_bytes(8))[0]
            # frame.payload_len = sum([ _d & (0xFF << (base * 8)) for base in range(4)])

        if frame.mask:
            self._on_frame_mask_key(self.recv_bytes(4), frame)
    
        self._on_frame_data(self.recv_bytes(frame.payload_len), frame)

        return frame

    def send_frame_data(self, frame):
        bin_frame = b''
        bin_frame += struct.pack(b'B', (frame.fin << 7)  + frame.opcode)
        payload_len = len(frame.data)

        if payload_len < 126:
            bin_frame += struct.pack(b'B', payload_len)

        elif payload_len <= 0xffff:
            bin_frame += struct.pack(b'B', 126)
            bin_frame += struct.pack(b'!H', payload_len)

        else:
            bin_frame += struct.pack(b'B', 127)
            bin_frame += struct.pack(b'!Q', payload_len)

        bin_frame += frame.data

        self.stream.send_string(bin_frame)

    def recv_bytes(self, size):
        if size == 0:
            return b''

        chunk = self.stream.recv(size)

        if not chunk:
            raise WebSocketError('connection closed')

        has_much = size - len(chunk) # 因为一次只能获取65536长度的数据，所以需要检测下有没有余下的
        if has_much > 0:
            chunk += self.recv_bytes(has_much)

        if len(chunk) != size:
            raise WebSocketError('connection closed')
        
        return chunk

    def _on_frame_data(self, _buffer, frame):
        if not frame.mask:
            frame['data'] = _buffer
            return

        _content = b''
        for i in range(frame.payload_len):
            _b = struct.unpack(b'!B', _buffer[i])[0]
            _content += struct.pack(b'!B', _b ^ frame.mask_key[i % 4])

        frame['data'] = _content

    def _on_frame_mask_key(self, _buffer, frame):
        frame['mask_key'] = struct.unpack(b'4B', _buffer)

    def _on_frame_header(self, _buffer, frame):
        _d = struct.unpack(b'BB', _buffer)
        frame['fin'] = _d[0] >> 7
        frame['opcode'] = _d[0] & 0xF
        frame['mask'] = _d[1] >> 7
        frame['payload_len'] = _d[1] & 0x7F

class WebSocketHandler(RequestHandler):
    @async
    def GET(self):
        request_error = self.check_request_header_error()
        if request_error:
            self.set_status(400)
            self.finish(request_error)
            return

        if not self.origin_is_accept():
            self.set_status(403)
            self.finish('access origin illegal')
            return

        self._websocket_conn = WebSocketConnection(self.__websocket_version,
                self.__websocket_key, self)
        try:
            self._websocket_conn.accept()
        except WebSocketError as ex:
            self._websocket_conn.close()

    def close(self):
        self._websocket_conn.close()

    def on_open(self):
        pass

    def on_close(self, status, reason):
        pass

    def on_message(self, frame):
        pass

    def on_ping(self, frame):
        frame = ObjectDict({'fin': 1, 'data': b'gale ws pong', 'opcode': 0xA})
        self._websocket_conn.send_frame_data(frame)

    def send_message(self, chunk):
        if not is_string(chunk):
            raise WebSocketError('message type must str or unicode')
        frame = ObjectDict({'fin': 1, 'data': utf8(chunk), 'opcode': 1})
        self._websocket_conn.send_frame_data(frame)

    def check_request_header_error(self):
        for header in ('upgrade', 'websocket_key', 'websocket_version'):
            _error = getattr(self, '_check_'  + header)()
            if _error:
                return _error

        return None

    @property
    def __websocket_key(self):
        return self.request.get_header('Sec-WebSocket-Key', '')

    @property
    def __websocket_version(self):
        _version = self.request.get_header('Sec-WebSocket-Version')
        return _version and int(_version) or None

    def _check_upgrade(self):
        upgrade = self.request.get_header('Upgrade', '')
        if upgrade.lower() != 'websocket':
            return 'upgrade only support websocket'

    def _check_websocket_version(self):
        _version = self.__websocket_version
        if (not _version) or (_version != 13):
            self.set_header('Sec-WebSocket-Version', '13')
            return 'currently support version == 13'

    def _check_websocket_key(self):
        if not self.__websocket_key:
            return 'missing websocket key'

    def origin_is_accept(self):
        origin = utf8(self.request.get_header('Origin', ''))
        if not origin:
            return False

        origin_host = urlsplit(origin).netloc
        origin_host = ':' in origin_host and origin_host or (origin_host + ':80')
        if origin_host != self.request.host + ':' + self.request.port:
            return False

        return True




#!/usr/bin/env python
#coding:utf8
# Author          : tuxpy
# Email           : q8886888@qq.com
# Last modified   : 2015-03-25 14:01:15
# Filename        : gale/config.py
# Description     : 
from __future__ import unicode_literals, print_function

CRLF = b"\r\n"




#!/usr/bin/env python
#coding:utf-8
# Author        : tuxpy # Email         : q8886888@qq.com.com
# Last modified : 2015-06-09 08:38:20
# Filename      : cache.py
# Description   : 
from collections import OrderedDict
from gale.e import CacheError
import time
import hashlib
from gale.escape import utf8
from functools import wraps
try:
    import cPickle as pickle
except ImportError:
    import pickle

class ICacheManager(object):
    def get(self, key): 
        """获取cache""" 
        raise NotImplementedError

    def set(self, key, value, expire = None):
        """设置一个cache,可传入key和expire"""
        raise NotImplementedError

    def flush_all(self):
        """清空所有缓存"""
        raise NotImplementedError

    def flush(self, key):
        """清删除某个key"""
        raise NotImplementedError

    @property
    def __name__(self):
        raise NotImplementedError

class MemCacheManager(ICacheManager):
    """基于内存的使用lru算法"""
    _cache_dict = OrderedDict() # 这是为了对那些不经常访问的数据做删除，所以要用到有序的dict
    _expire_dict = OrderedDict()  # 存过期时间的map
    def __init__(self, expire = 7200, max_size = 100):
        self.expire = expire
        self.max_size = max_size

    def set(self, key, value, expire = None):
        self.flush(key)
        expire = expire or self.expire
        assert isinstance(expire, int)
        self._cache_dict[key] = value
        self._expire_dict[key] = int(time.time()) + expire
        self.clearup()
        
    def flush(self, key):
        if key in self._cache_dict:
            del self._cache_dict[key]
            del self._expire_dict[key]

    def flush_all(self):
        self._cache_dict.clear()
        self._expire_dict.clear()

    def get(self, key):
        self.clearup()
        return self._cache_dict.get(key, None)

    def ttl(self, key):
        ttl = self._expire_dict.get(key) or 0
        return ttl

    def clearup(self):
        now = int(time.time())
        for key in self._expire_dict:
            if now > self._expire_dict[key]:
                self.flush(key)
            else:
                break # 因为是根据插入的次序排的，有一个不过期，就说明后面的都不会过期了。

        while len(self._cache_dict) > self.max_size:
            for key in self._cache_dict:
                self.flush(key)
                break

    @property
    def __name__(self):
        return 'mem'

class RedisCacheManager(ICacheManager):
    def __init__(self, db, expire = 7200, prefix = None):
        self.expire = expire
        self.db = db
        self.prefix = prefix or 'gale:redis_cache:'

    def set(self, key, value, expire = None):
        key = self.prefix + utf8(key)
        expire = expire or self.expire
        value = pickle.dumps(value, protocol = 1)

        pipe = self.db.pipeline()
        pipe.set(key, value)
        pipe.expire(key, expire)
        result_list = pipe.execute()

        for result in result_list:
            assert result

    def get(self, key):
        key = self.prefix + utf8(key)
        value = self.db.get(key)
        if not value:
            return None

        value = pickle.loads(value)

        return value

    def ttl(self, key):
        key = self.prefix + utf8(key)

        return self.db.ttl(key) or 0

    def flush_all(self):
        pipe = self.db.pipeline()
        for key in self.db.keys(self.prefix + '*'):
            pipe.delete(key)

        pipe.execute()

    def flush(self, key):
        self.db.delete(key)

    @property
    def __name__(self):
        return 'redis'

def _generate_key(key, args, kwargs):
    m = hashlib.md5(utf8(key))
    [m.update(utf8(str(arg))) for arg in args]
    [m.update(utf8("%s=%s" % tuple(item))) for item in kwargs.items()]

    return m.hexdigest()

def cache(key = None, expire = None, on = None):
    """on表示cache存在哪，可选mem,和redis，如果没有选，则默认"""
    def outer_wrap(func):
        def inner_wrap(hdl, *args, **kwargs):
            cache_manager = hdl.get_cache_manager(on)
            _key = key or (func.__name__ + hdl.__class__.__name__ + hdl.__module__)
            _key = _generate_key(_key, args, kwargs)
            cache = cache_manager.get(_key)
            if cache:
                return cache

            value = func(hdl, *args, **kwargs)
            cache_manager.set(_key, value, expire)

            return value

        return inner_wrap

    return outer_wrap

def page(expire = None, on = None):
    def outer_wrap(func):
        @wraps(func)
        def inner_wrap(hdl, *args, **kwargs):
            if hdl.request.method != 'GET':
                raise CacheError('page cache only support GET method')
            cache_manager = hdl.get_cache_manager(on)
            _key = hdl.request.uri
            cache = cache_manager.get(_key)
            if cache and cache['status'] in (200, 304):
                hds = cache['headers']
                hdl.set_header('Content-Type', hds['Content-Type'])
                hdl.set_header('Server', 'Gale Cache')
                hdl.set_status(200)
                hdl.push(cache['body'])
                return 

            hdl.cache_page = True
            result = func(hdl, *args, **kwargs)
            hdl.flush()
            if hdl.response_body:  # 只在200和304时有body
                cache = {'status': hdl._status_code, 
                        'headers': hdl._headers, 'body': hdl.response_body}
                cache_manager.set(_key, cache, expire)

            return result

        return inner_wrap

    return outer_wrap

def cache_self(func):
    @wraps(func)
    def wrap(self):
        _key = '_cache_' + func.__name__[0]
        _cache = getattr(self, _key, None)
        if _cache:
            return _cache

        _value = func(self)
        setattr(self, _key, _value)

        return _value

    return wrap




#!/usr/bin/env python
#coding:utf8
# Author          : tuxpy
# Email           : q8886888@qq.com
# Last modified   : 2015-03-26 12:55:47
# Filename        : gale/socket.py
# Description     : 
from __future__ import unicode_literals
import gevent
from gevent import socket

class IOSocket():
    def __init__(self, socket, max_buff= 4096):
        self._socket = socket
        self.max_buff = max_buff
        self.closed = False
        self._buff = b''
        self.on_close_callback = None

    def write(self, chunk):
        self._socket.sendall(chunk)

    def clone(self):
        return IOSocket(self._socket, self.max_buff)

    def gevent_exception(self, *args, **kwargs):
        self.close()

    def close(self):
        if self.closed:
            return
        self.on_close()
        self._socket.close()
        self.closed = True

    def set_timeout(self, secs = 60):
        self._socket.settimeout(secs)

    def on_close(self): # 在RequestHandler中会被赋值
        if self.on_close_callback:
            self.on_close_callback()

    def send_string(self, string):
        try:
            self._socket.sendall(string)
        except Exception as e:
            self.close()

    def is_close(self):
        if self.closed:
            return True
        try:
            if (self._socket.getpeername()): # 如果 closed是False,为了避免有时socket会异常断开，就再判断一下
                return False
        except:
            return True

    def recv(self, buffer_size):
        try:
            chunk = self._socket.recv(buffer_size)
            return chunk
        except Exception as ex:
            return None




#!/usr/bin/env python
#coding:utf-8
# Author        : tuxpy
# Email         : q8886888@qq.com.com
# Last modified : 2015-03-30 18:06:49
# Filename      : log.py
# Description   : 
from __future__ import unicode_literals
import logging
import re

access_log = logging.getLogger('access_log')
gen_log = logging.getLogger('gen_log')

class ColorLogFormatter(logging.Formatter):
    COLOR_CODE = {logging.INFO: '32',
            logging.WARNING: '33',
            logging.ERROR: '31'}

    def __init__(self, fmt = None, datefmt = None):
        logging.Formatter.__init__(self, fmt, datefmt)
        self._raw_fmt = fmt
        self._re_fmt = re.compile(r'(%\(\w+?\)\w)')

    def format(self, record):
        def generate_color_str(m):
            color_str = "\x1b[0;{color};1m{arg}\x1b[0m".format(color = self.COLOR_CODE[record.levelno], arg = m.group(1))
            return color_str

        self._fmt = self._re_fmt.sub(generate_color_str, 
                self._raw_fmt, count = 1)
        return logging.Formatter.format(self, record)

def config_logging(log_settings):
    """配置logging"""
    save_file = log_settings.get('file')
    stream_handler = logging.StreamHandler()
    handler_list = [stream_handler]
    if save_file:
        file_handler = logging.FileHandler(save_file)
        handler_list.append(file_handler)
    for _handler in handler_list:
        _handler.setLevel(getattr(logging, log_settings.get('level', 'DEBUG'),
            logging.DEBUG))
        formatter_class = isinstance(_handler, logging.StreamHandler) and ColorLogFormatter or logging.Formatter
        formatter = formatter_class('%(asctime)s %(filename)s [%(process)d] %(levelname)s %(message)s',
                datefmt = log_settings.get('datefmt', '%Y-%m-%d %H:%M:%S'))

        _handler.setFormatter(formatter)
        access_log.addHandler(_handler)
    access_log.setLevel(stream_handler.level) # 如果只是addHandler的话access_log的日志等级是没有被设置过的，也就默认的WARNING
    gen_log.setLevel(stream_handler.level) 

    if 'request_format' in log_settings:
        access_log.request_format = log_settings['request_format']

def generate_request_log(handler):
    log_format_string = getattr(access_log, 'request_format',
            '{start_timestamp} {first_line} {status_code} {client_ip} {request_time}')

    request = handler.request

    return log_format_string.format(
            start_timestamp = request.start_time,
            method = request.method,
            uri = request.uri,
            path = request.path,
            client_ip = request.client_ip,
            status_code = handler.get_status(),
            first_line = request.first_line,
            version = request.version,
            body_size = request.size,
            response_size = handler.get_been_set_header('Content-Length'),
            request_time = "%.2fms" % (request.request_time * 1000, ),
            )




#!/usr/bin/env python
#coding:utf-8
# Author        : tuxpy
# Email         : q8886888@qq.com.com
# Last modified : 2015-05-02 12:18:07
# Filename      : version.py
# Description   : 
import sys
is_py3 = sys.version[0] == '3'





#!/usr/bin/env python
#coding:utf-8
# Author        : tuxpy
# Email         : q8886888@qq.com.com
# Last modified : 2015-06-09 10:04:55
# Filename      : setup.py
# Description   : 
from setuptools import setup
from gale import version
setup(
        name = 'gale_web',
        version = version,
        author = 'tuxpy',
        url = 'https://github.com/lujinda/gale',
        include_package_data = True,
        packages = [
            'gale',
            'gale.wsgi',
            ],
        install_requires = ['gevent', 'msgpack-python'],
        description = 'a web framework like tornado',
        )





#!/usr/bin/env python
#coding:utf8
# Author          : tuxpy
# Email           : q8886888@qq.com
# Last modified   : 2015-03-26 13:16:10
# Filename        : gale/http.py
# Description     : 
from gale.config import CRLF
from gale.e import HeaderFormatError, NotSupportHttpVersion, JSONError
from gale.utils import urlsplit, urldecode
from gale.log import gen_log
from gale import escape
from gale.escape import native_str
from gale import cache
from time import time
import re
import json


class HTTPRequest():
    def __init__(self, method, uri, version, headers, body, connection, real_ip = True):
        self.method = native_str(method)
        self.uri = native_str(uri)
        self.version = native_str(version)
        self.version_num = self.__version_num() 
        self.headers = headers
        self.body = body
        _urlparse = urlsplit(self.uri)
        self.path = _urlparse.path
        self.host = headers.get('Host', '').strip()
        self.host, self.port = (self.host.split(':') + ['80'])[:2]
        self.query = _urlparse.query
        self.connection = connection
        self._start_time = time()
        self.real_ip = real_ip
        self.cookies = self.__get_cookies()
        self.files = {}
        self._body_arguments = {}
        self.client_ip = self.__remote_ip()
        self.client_port = self.connection.remote_port()

    @property
    def size(self):
        """return requesty body size"""
        return len(self.body)

    @property
    def first_line(self):
        return "{method} {uri} {version}".format(
                method = self.method, uri = self.uri, version = self.version)

    def _parse_body(self):
        content_type = self.headers.get(u'Content-Type', '')

        try:
            if content_type.startswith('multipart/form-data'):
                parse_multipart_form_data(content_type, self.body, args = self._body_arguments,
                        files = self.files)
            else:
                self._body_arguments = urldecode(self.body) 
        except Exception:
            self._body_arguments = {}

    @cache.cache_self
    def json(self):
        try:
            json_body = json.loads(self.body)
        except ValueError:
            raise JSONError

        return json_body

    def __get_cookies(self):
        _cookies = {}
        _cookies_string = self.headers.get('Cookie', '')
        if not _cookies_string:
            return {}

        _cookies_list = [_cookie_string.split('=', 1) for _cookie_string in  _cookies_string.split(';')]
        for _cookie_name, _cookie_value in _cookies_list:
            _cookies[_cookie_name.strip()] = _cookie_value.strip()

        return _cookies

    def __version_num(self):
        _version_num = self.version.split('/', 1)[1]
        if not re.match(r'^1\.[01]$', _version_num):
            raise NotSupportHttpVersion

        return _version_num

    def is_keep_alive(self):
        _connection_param = self.headers.get('Connection', '').lower()
        if _connection_param:
            return _connection_param.lower() != 'close'

        if self.version_num == '1.1': # 如果http版本是1.1默认是保持连接的
            return True

        return False

    def __remote_ip(self):
        if self.real_ip: 
            _remote_ip = self.headers.get("X-Real-IP", 
                self.headers.get("X-Forwarded-For", self.connection.remote_ip()))
        else:
            _remote_ip = self.connection.remote_ip()

        return _remote_ip

    @property
    def start_time(self):
        return self._start_time

    @property
    def request_time(self):
        return time() - self._start_time

    def get_header(self, name, default = None):
        """获取http header，不区分大小写, 并且Referrer和Referer都是一样的"""
        name = name.title()
        if name == 'Referrer':
            name = 'Referer'

        return self.headers.get(name, default)

    @property
    def query_arguments(self):
        return urldecode(self.query)

    @property
    def body_arguments(self):
        return self._body_arguments

    @property
    def all_arguments(self):
        _args = {}
        _args.update(self.query_arguments)
        _args.update(self.body_arguments)
        return _args

class HTTPHeaders(dict):
    def __init__(self, headers=None, is_request = False):
        dict.__init__(self)
        self.__is_request = is_request
        headers = headers or {}
        self._headers_map_list = {} # 数据格式: {name: [value1, value2,...]}

        if not isinstance(headers, dict): # 如果传入的headers不是字典，表示这是请求headers，则调用dict原有的__setitem__ 将头信息传进去，如果传入是dict，则表示是响应头，则需要处理一下，比如同域名的，但是不同域值要放一块
            self.__is_request = True
            headers = self._parse_headers(headers)
            for _k, _v in headers.items():
                dict.__setitem__(self, _k.title(), _v)

        for _k, _v in headers.items():
            self[_k] = _v

    @staticmethod
    def _parse_headers(headers):
        """把http头信息组织到一个字典中去"""
        _headers = {}
        _last_key = None
        headers = native_str(headers)
        assert not isinstance(headers, dict)
        for header_line in headers.split('\n'):
            header_line = header_line.rstrip('\r')
            if header_line.startswith('\x20') or header_line.startswith('\t') :  # 因为http首部是允许换行的, 所以如果这一行是以空格或制表符开头的，需要将信息加到之前那行
                _headers[_last_key] += ' ' + header_line.lstrip()
                continue
            else:
                _header_name, _header_value = header_line.split(':', 1)
            _headers[_header_name] = _header_value.strip()
            _last_key = _header_name

        return _headers

    def get_response_headers_string(self, first_line):
        """生成响应头，并加上最后的两个CRLF"""
        _headers_string = escape.utf8(first_line)
        for _name, _value_list in self._headers_map_list.items():
            for _value in _value_list:
                _header_line = "{name}: {value}".format(
                    name = _name, value = _value)
                _headers_string += (CRLF + escape.utf8(_header_line))

        return _headers_string + CRLF * 2


    def get_headers_items(self):
        items = []
        for key, values in self._headers_map_list.items():
            for value in values:
                items.append((key, value))
        return items

    def __setitem__(self, name, value):
        self._headers_map_list[name] = [value]

    def clear(self):
        self._headers_map_list.clear()

    def __getitem__(self, name):
        if name in self._headers_map_list:
            return self._headers_map_list[name][0]
        else:
            raise KeyError

    def set_default_header(self, name, value):
        if name not in self._headers_map_list:
            self[name] = value

    def get(self, name, default = None):
        if self.__is_request:
            return dict.get(self, name, default)
        if name not in self._headers_map_list:
            return default

        return self.__getitem__(name)

    def add(self, name, value):
        self._headers_map_list.setdefault(name, []).append(value)


class HTTPFile(object):
    def __init__(self, filename, body, content_type):
        self.filename = filename
        self.body = body
        self.content_type = content_type

    def size(self):
        return len(self.body)

    def __setitem__(self, name, value):
        return setattr(self, name, value)

    def __getitem__(self, name):
        return getattr(self, name)

def parse_content_disposition(con_dis):
    """解析multipart中的数据头部中的Content-Dispoition字段"""
    _con_dis = {}
    _key = ''
    for _field in con_dis.split(b'; '):
        if b'=' not in _field:
            continue
        key, value = _field.split(b'=', 1)
        value = value[1: -1] # 去掉两边的绰号
        _con_dis[key] = value

    return _con_dis

def parse_multipart_form_data(content_type, body, args, files):
    boundary = escape.utf8(content_type.split('boundary=')[-1])
    end_index = body.rfind(b'--' + boundary + b'--')
    if end_index == -1: # 如果没有结尾符的话，则说明这个主体格式是错误的
        gen_log.error('Invalid multipart/form-data: no final boundary')
        return

    parts = body[:end_index].split(b'--' + boundary + CRLF)
    for _part in parts:
        if not _part:
            continue
        _args_part, _body_part = _part.split(CRLF * 2)[:2]
        body = _body_part[:-2]
        headers = HTTPHeaders._parse_headers(_args_part) # 这里的headers是指某一个数据块的头部信息
        con_dis = headers.get(b'Content-Disposition', None)
        if not con_dis:
            gen_log.error('must have Content-Disposition')
            return

        con_dis = parse_content_disposition(con_dis)

        name = escape.to_unicode(con_dis[b'name'])
        if b'filename' in con_dis: # 如果有filename，则表示这是一个文件
            filename = con_dis.get(b'filename')
            if not filename:
                continue

            files.setdefault(name, []).append(
                    HTTPFile(filename = escape.to_unicode(filename), body = body,
                            content_type = escape.to_unicode(headers.get(b'Content-Type', 'application/octet-stream'))))
        else:
            args.setdefault(name, []).append(escape.to_unicode(body))




#!/usr/bin/env python
#coding:utf8 # Author          : tuxpy
# Email           : q8886888@qq.com
# Last modified   : 2015-03-26 13:16:41
# Filename        : gale/web.py
# Description     : 
from __future__ import unicode_literals, print_function
try: 
    import Cookie # py2 
except ImportError:
    import http.cookies as Cookie

from gale import template, cache
from gale.http import  HTTPHeaders
from gale.e import HasFinished, NotSupportMethod, ErrorStatusCode, MissArgument, HTTPError, LoginHandlerNotExists, LocalPathNotExist, CookieError, CacheError
from gale.utils import  parse_request_range, single_pattern, is_string, urlsplit, urlquote, urlunquote, ShareDict, made_uuid, get_mime_type, format_timestamp, code_mess_map # 存的是http响应代码与信息的映射关系
from gale.escape import utf8, param_decode, native_str, to_unicode
from gale.log import (access_log, config_logging, generate_request_log)
from gale.ipc import IPCDict
from gale.session import FileSessionManager
from gale.restapi import RestApi

import traceback
import time
from functools import wraps
import hashlib
import hmac 
import os 
import json
import glob
import gzip
import re
import base64
import gevent
import types

try:
    from cStringIO import StringIO as s_io
except ImportError:
    try:
        from StringIO import StringIO as s_io
    except ImportError:  #  py3
        from io import BytesIO as s_io

try:
    import urlparse # py2
except ImportError:
    import urllib.parse as urlparse # py3

_ALL_METHOD = ('POST', 'GET', 'PUT', 'DELETE', 'HEAD', 'PATCH')
re_signed_cookie = re.compile(r'^\|\d+')

BUFFER_SIZE = 4096 * 10

def auth_401(method):
    @wraps(method)
    def wrap(self, *args, **kwargs):
        if self.current_user:
            return method(self, *args, **kwargs)
        else:
            self.set_header('WWW-Authenticate', "Basic realm='Please input'")
            raise HTTPError(401)

    return wrap


class ResponseBody(object):
    """This is RequestHandler.push syntactic sugar"""
    def __set__(self, handler, value):
        self.handler._push_buffer = []
        self.handler.push(value)

    def __get__(self, handler, handler_class):
        self.handler = handler
        return self

    def __add__(self, value):
        self.handler.push(value)

class ResponseBodyBuffer(list):
    def __add__(self, _buffer):
        print(_buffer)

class RequestHandler(object):
    """主要类，在这里完成对用户的请求处理并返回"""
    def __init__(self, application, request, kwargs = None):
        self.kwargs = kwargs or {}
        self.application = application
        self.request = request
        self.cache_page = False
        self.response_body = None
        self._finished = False
        self._headers = HTTPHeaders()
        self._buffer_md5 = hashlib.md5()
        self.query = ParamsObject(self.request.query_arguments)
        self._body = BodyParamsResponseObject(self.request.body_arguments)
        self.__been_writen_headers = False
        self.__is_sending_file = True # 表示正在发送文件， flush_body的方式会改变
        self.request.connection.on_close_callback = self.on_connect_close

        if self.request.method not in _ALL_METHOD:
            raise NotSupportMethod

        self.init_data()
        self.init_headers()

    @property
    def body(self):
        return self._body

    @body.setter
    def body(self, _buffer):
        self._body.set_buffer(_buffer)

    def init_data(self):
        """一些初始化工作都可以在这做"""
        pass

    def init_headers(self): 
        """设置response的headers信息"""
        self.set_headers({
            'Content-Type'  :       'text/plain; charset=utf-8',
            'Server'        :       'Gale Server',
            'Date'          :       format_timestamp(),
            })
        self.set_status(200)

    @property
    def allow_methods(self):
        return [ method for method in _ALL_METHOD if hasattr(self, method)]

    def set_default_headers(self):
        """可以在这里设置一些默认的headers信息(使用set_header[s])"""
        pass

    def ALL(self, *args, **kwargs):
        pass

    """
    # 当http request到来时，会自动调用Handler下的对应方法
    def HEAD(self, *args, **kwargs):
        raise NotSupportMethod

    def GET(self, *args, **kwargs):
        raise NotSupportMethod

    def POST(self, *args, **kwargs):
        raise NotSupportMethod

    def DELETE(self, *args, **kwargs):
        raise NotSupportMethod

    def PUT(self, *args, **kwargs):
        raise NotSupportMethod
    """

    @property
    def server_settings(self):
        return self.application.server_settings

    @property
    def login_url(self):
        return self.settings.get('login_url')

    @property
    def current_user(self):
        return self.get_current_user()

    def _generate_abspath(self, *p):
        return os.path.join(os.path.dirname(os.path.abspath(__file__)),
                *p)

    def get_current_user(self):
        """如果需要实现用户验证，请在此完成，登录成功返回一个非False的值就行了，如果返回的是False，则表示验证失败，会被转到登录url上。"""
        return None

    def push(self, _buffer):
        if self.is_finished:
            raise HasFinished('can\' push finished after')

        if isinstance(_buffer, dict):
            _buffer = json.dumps(_buffer)
            self.set_header('Content-Type', 'application/json')

        _buffer = utf8(_buffer)
        if self.settings.get('dynamic_304', True):
            self._buffer_md5.update(_buffer)
        self.body + _buffer

    def is_supported_http1_1(self):
        return self.request.version == 'HTTP/1.1'

    @property
    def static_path(self):
        return self.settings.get('static_path')

    def redirect(self, url, temp = True, status_code = None):
        if not status_code:
            status_code = temp and 302 or 301 # 如果没有指定 status_code， 则根据是否是临时重定向来决定code

        if not self.body.empty():
            raise Exception("Can't redirect after push")
        assert isinstance(status_code ,int) and (300 <= status_code <= 399)
        self.set_status(status_code)

        self.set_header('Location', urlparse.urljoin(utf8(self.request.uri), utf8(url)))

    def render(self, template_name, **kwargs):
        if self.is_finished:
            return False
        html = to_unicode(self.render_string(template_name, **kwargs))

        # 自动加载js
        load_js_list = self.__get_load_list('js')
        if load_js_list:
            offset = html.rfind('</body>')
            js_html = '\n'.join(['<script src="{js_path}"></script>'.format(js_path = to_unicode(js_path)) for js_path in load_js_list])

            html = html[:offset] + js_html + '\n' + html[offset:]

        # 自动加载css
        load_css_list = self.__get_load_list('css')
        if load_css_list:
            offset = html.rfind('</head>')
            css_html = '\n'.join(['<link rel="stylesheet" href="{css_path}" />'.format(css_path = to_unicode(css_path)) for css_path in load_css_list])
            html = html[:offset] + css_html + '\n' + html[offset:]

        self.set_header('Content-Type', 'text/html;charset=UTF-8')
        self.push(html)

    def __get_load_list(self, _type):
        assert _type in ('js', 'css')

        _list = []
        load_list = getattr(self, 'load_' + _type)()
        if not load_list:
            return []

        load_list = isinstance(load_list, (list, tuple)) and load_list or [load_list]

        for _item in load_list:
            _abs_path = self.get_static_url(_item, is_abs = True)
            if os.path.exists(_abs_path) == False:
                raise OSError('%s not exist' % (_abs_path, ))

            if os.path.isdir(_abs_path):
                _list.extend([_url for _url in self.get_static_urls(_item) \
                        if _url.endswith('.' + _type)]) # 当css时，过滤掉非css的，js时，过滤掉非js的
            else:
                _list.append(self.get_static_url(_item))

        return _list

    def load_js(self):
        return []

    def load_css(self):
        return []

    def render_string(self, template_name, **kwargs):
        for _param_key in kwargs:
            kwargs[_param_key] = native_str(kwargs[_param_key])

        _template_loader = self.application._template_cache.get(template_name)
        if not _template_loader:
            _template_loader = self.create_template_loader(self.get_template_path())
            if self.settings.get('debug') != True:
                self.application._template_cache[template_name] = _template_loader

        t = _template_loader.load(template_name)
        name_space = self.get_name_space()
        kwargs.update(name_space)
        kwargs['module'] = self._load_ui_module
        
        return t.generate(**kwargs)

    def string_render(self, string, **kwargs):
        _template_loader = self.create_template_loader()
        name_space = self.get_name_space()
        kwargs.update(name_space)
        t = _template_loader.load(string)
        return t.generate(**kwargs)

    def send_file(self, attr_path, attr_name = None, charset = 'utf-8', speed = None, is_attr = True):
        """speed: 下载速度, 单位是字节B"""
        if not self.body.empty():
            raise Exception("Can't redirect after push")

        if not os.path.isfile(attr_path):
            raise OSError('file: %s  not found' % (attr_path, ))


        attr_name = attr_name or os.path.basename(attr_path)
        self.set_header('Content-Type', get_mime_type(attr_path))
        if is_attr:
            self.set_header('Content-Disposition', 
                    native_str('attachment;filename="%s"') % (native_str(attr_name), ))
            self.set_header('Accept-Ranges', 'bytes')

        self.__is_sending_file = True

        sleep_secs = speed and (1.0 / (1.0 * speed / BUFFER_SIZE)) or None

        file_size = os.stat(attr_path).st_size
        read_range = self.request.get_header('Range', None)
        start, _ = parse_request_range(read_range) # 由于是断点续传，只考虑start
        if start > 0:
            self.set_header('Content-Range', 
                    'bytes %s-%s/%s' % (start, file_size - 1, file_size))
            self.set_status(206)
        self.set_header('Content-Length', file_size - start)
        
        self.__send_file(attr_path, sleep_secs, start = start)

        self.finish()

    def __send_file(self, attr_path, sleep_secs = None, start = 0):
        with open(attr_path, 'rb') as fd:
            fd.seek(start)
            while not self.request.connection.closed:
                _content = fd.read(BUFFER_SIZE)
                if not _content:
                    break;
                self.push(_content)
                self.flush()
                if sleep_secs:
                    gevent.sleep(sleep_secs)

            self.body.flush_buffer()

    def create_template_loader(self, template_path = None):
        return template_path and template.Loader(template_path) or template.StringLoader()

    def get_template_path(self):
        return self.settings.get('template_path', 'template')

    def get_name_space(self):
        """一些可以在模块中用的变量或方法"""
        name_space = {
                'client_ip' :   self.client_ip,
                'handler'   :   self,
                'request'   :   self.request, 
                'static_url'    :   self.get_static_url,
                '_tt_modules'   :   _UINameSpace(self),
                'current_user'  :   self.get_current_user,
                }

        ext_name_space = self.get_ext_name_space()
        if not isinstance(ext_name_space, dict):
            raise TypeError
        name_space.update(ext_name_space)
        return name_space

    def get_ext_name_space(self):
        """可以在这里定义一些额外的namespce"""
        return {}

    def get_static_url(self, file_path, is_abs = False):
        assert self.static_path, "static_url must had 'static_path' in app\'s settings"
        static_class = self.settings.get('static_class', StaticFileHandler)
        if is_abs:
            return static_class.make_absolute_path(self.static_path, file_path)
        static_url = static_class.get_static_url(self.settings, file_path)
        return static_url

    def get_static_urls(self, path):
        """获取某一个目录下的所有静态资源"""
        assert self.static_path, "get_static_urls must had 'static_path' in app\'s settings"
        abs_path = self.get_static_url(path, is_abs = True)
        if not os.path.isdir(abs_path):
            return [abs_path]

        urls = []
        for dirname, _, _files  in os.walk(abs_path):
            for _file in _files:
                url = os.path.join(dirname, _file)[len(self.static_path) + 1: ]
                urls.append(self.get_static_url(url))

        return urls

    @property
    def is_finished(self):
        return self._finished

    def process_buffer(self, _buffer):
        if self.settings.get('gzip', False):
            processor = GzipProcessor(self.request.headers)
            return processor.process(_buffer, self._headers, 
                    self.application._buffer_stringio)

        return _buffer

    def should_return_304(self, buffer_md5):
        if self.get_status() not in (200, 304):
            return False

        if self.request.method != 'GET':
            return False

        dynamic_304 = self.settings.get('dynamic_304', True)
        if dynamic_304 == False or self.request.get_header('Cache-Control') == 'no-cache':
            return False

        request_version = self.request.get_header('If-None-Match', '')
        if not request_version:
            return False

        return request_version == buffer_md5

    def _flush_body(self, response_body):
        connection = self.request.connection
        if self.__is_sending_file:
            if response_body:
                connection.send_body(response_body)
            else:
                connection.send_finish_tag()
            return

        _buffer_md5 = self._buffer_md5.hexdigest()
        self.set_header('Etag', _buffer_md5)

        is_return_304 = self._status_code == 304 or self.should_return_304(_buffer_md5)
        if is_return_304:
            self.set_status(304)

        if is_return_304 != True and self.request.method != 'HEAD': # 如果是HEAD请求的话则不返回主体内容
            connection.send_body(response_body)
            connection.send_finish_tag()

    def flush(self, _buffer = None):
        self.before_flush()
        if self.is_finished:
            return

        if self.request.connection.closed:
            return

        _buffer = _buffer or self.body.to_bytes()
        _response_body = self.process_buffer(_buffer)

        if self.__been_writen_headers == False:
            self.__been_writen_headers = True
            self.set_header('Connection', self.request.is_keep_alive() and 'Keep-Alive' or 'Close')
            self.set_default_header('Content-Length', len(_response_body))
            if hasattr(self, '_new_cookie'):
                for cookie in self._new_cookie.values():
                    self.add_header('Set-Cookie', cookie.OutputString())
            _headers = self._headers.get_response_headers_string(self.response_first_line) # 把http头信息生成字符串
            self.request.connection.send_headers(_headers)

        self._flush_body(_response_body)
        self.body.flush_buffer()

        if self.cache_page and self._status_code in (200, 304):
            self.response_body = _buffer
        else:
            self.response_body = None

        del _buffer

    def log_print(self):
        _log = generate_request_log(self)

        # 根据不同的http状态代码来输出不同的日志 
        if self._status_code < 400:
            access_log.info(_log)
        elif self._status_code < 500:
            access_log.warning(_log)
        else:
            access_log.error(_log)

    @property
    def client_ip(self):
        return self.request.client_ip

    def on_referrer_error(self):
        raise HTTPError(403)

    def on_finish(self):
        """会在执行完finish时执行"""
        pass

    def before_flush(self):
        """在刷新前做点事"""
        pass

    def finish(self, chunk = None):
        if self._finished:
            return
        if not self.is_finished:
            self.flush(chunk)

        if not self.request.is_keep_alive():
            self.request.connection.close()

        self._finished = True # 不管是不是keep alive，都需要把它设置成True，因为连接跟这个没关系，每一次有新的请求时，都会生成一个新的RequestHandler
        self.log_print()
        self.on_finish()

    def set_default_header(self, name, value):
        self._headers.set_default_header(name, value)

    def get_been_set_header(self, name, default = ''):
        try:
            return self._headers[name]
        except KeyError:
            return default

    def set_header(self, name, value):
        if value == None:
            return
        self._headers[name] = value

    def clear_headers(self):
        self._headers.clear()

    def add_header(self, name, value):
        self._headers.add(name, value)

    def set_headers(self, headers):
        if not isinstance(headers, dict):
            raise TypeError
        for _k, _v in headers.items():
            self.set_header(_k, _v)

    @property
    def cookies(self):
        """本次请求中所有的cookies"""
        return self.request.cookies

    def set_cookie(self, name, value, domain=None, expires=None, path = '/', expires_day = None, **kwargs):
        name, value = native_str(name), native_str(value)
        self._new_cookie = getattr(self, '_new_cookie', Cookie.SimpleCookie())
        self._new_cookie.pop(name, None) # 如果已经存在了，则删除它

        self._new_cookie[name] = value
        morsel = self._new_cookie[name]
        if domain:
            morsel['domain'] = domain

        if path:
            morsel['path'] = path

        if expires_day and (not expires):
            expires = expires_day * 60 * 60 * 24

        if expires:
            morsel['expires'] = format_timestamp(time.time() + expires)

        for k, v in kwargs.items():
            if k == 'max_age':
                k = 'max-age'
            morsel[k] = v

    def clear_cookie(self, name, path = '/', domain = None):
        self.set_cookie(name, value = '', 
                path = path, domain = domain, expires = -100)

    def clear_all_cookies(self, path = '/', domain = None):
        for _cookie_name in self.cookies:
            self.clear_cookie(name, path = path, domain = domain)

    def get_cookie(self, cookie_name, default = None):
        value = self.cookies.get(cookie_name, default)
        if value and value[0] == '\"' and value[-1] == '\"':
            return value[1: -1]
        else:
            return value

    def get_signed_cookie(self, cookie_name, default = None):
        secret = self.settings.get('cookie_secret', None)
        if not secret:
            raise CookieError("use signed cookie app\'s settings must has 'cookie_secret'")
        value = self.get_cookie(cookie_name)
        if value == None:
            return default

        if (not re_signed_cookie.match(value)) and self.cookie_support_tornado == False:
            return None

        if self.cookie_support_tornado:
            return decode_signed_cookie_tornado(secret, cookie_name, value)
        else:
            return decode_signed_cookie(secret, cookie_name, value)

    def set_signed_cookie(self, name, value, *args, **kwargs):
        secret = self.settings.get('cookie_secret', None)
        if not secret:
            raise CookieError("use signed cookie app\'s settings must has 'cookie_secret'")

        signed_value = encode_signed_cookie(secret, name, value)
        self.set_cookie(name, signed_value, *args, **kwargs)

    @property
    def cookie_support_tornado(self):
        return bool(self.settings.get('cookie_support_tornado', False))

    def set_status(self, status_code = 200, status_message = None):
        self.status_message = status_message # 可以自定义状态代码描述
        self._status_code = status_code

    def get_status(self):
        return self._status_code

    def process_raise_error(self, e):
        """当出现错误的时候会被调用"""
        _status_code = getattr(e, 'status_code', 500)
        _status_mess = getattr(e, 'status_mess', None)
        self.set_status(_status_code, _status_mess)
        self.push_error() # 调用错误处理函数

    def push_error(self):
        """处理http异常错误"""
        if self.is_finished:
            return
        self.send_error(traceback.format_exc())

    def raise_error(self, status_code = 500, status_msg = None):
        raise HTTPError(status_code, status_msg)

    def send_error(self, exc):
        """把异常信息推送出去"""
        exc = utf8(exc)
        self.body.flush_buffer()
        self.push(self.response_first_line)
        if not self.settings.get('debug', False): # 只允许 在debug情况下输出错误
            return

        if self.get_status() >= 500:
            self.push(exc + b'\n')
        else:
            pass

    @property
    def _status_mess(self):
        return self.status_message or code_mess_map.get(self._status_code) # 如果没有自定义状态代码描述的话，就根据标准来
    
    @property
    def response_first_line(self):
        _message = self._status_mess
        if not _message:
            raise ErrorStatusCode
        return "{version} {code} {message}".format(
                version = self.request.version, code = self.get_status(),
                message = _message)

    def get_query_arguments(self, param):
        return self.request.query_arguments.get(param, [])

    def get_body_arguments(self, param):
        return self.request.body_arguments.get(param, [])

    def get_query_argument(self, param, default = None):
        _args = self.get_query_arguments(param)
        return self.__get_one_argument(_args, default)

    def get_body_argument(self, param, default = None):
        _args = self.get_body_arguments(param)
        return self.__get_one_argument(_args, default)

    def get_argument(self, param, default = None):
        _args = self.request.all_arguments.get(param, [])
        return self.__get_one_argument(_args, default)

    def get_arguments(self, param):
        return self.request.all_arguments.get(param, [])

    def get_files(self, name):
        return self.request.files.get(name, [])

    def get_file(self, name):
        return self.__get_one_argument(self.get_files(name))

    @property
    def session(self):
        _session = getattr(self, '_session', None)
        if _session != None:
            return _session

        from gale.session import Session

        _session_manager = getattr(self.application, 'session_manager', None)

        if not _session_manager:
            _session_manager = FileSessionManager(session_secret = 'galesessionsecret' ,
                        session_timeout = 86400 * 60) # 如果application中没有配置session_manager，则自动生成一个FileSessionManager，推荐想用session的请自己在application中指定 session_manager
            setattr(self.application, 'session_manager', _session_manager)

        _session = Session(_session_manager, self)

        setattr(self, '_session', _session)

        return _session

    def __get_one_argument(self, args, default = []):
        if (args == []) and (default != []): # 如果没有参数，但是指定了默认值，就返回默认的
            return default

        if args == []:
            raise MissArgument

        return args[0]

    @property
    def settings(self):
        return self.application.settings

    @property
    def ui_settings(self):
        return self.application.ui_settings

    def _load_ui_module(self, module_name, *args, **kwargs):
        _module = self.ui_settings.get(module_name)
        assert issubclass(_module, UIModule), 'module must extend from UIModule'
        _module_instance = _module(self)
        return _module_instance.render(*args, **kwargs)

    def get_cache_manager(self, on = None):
        s_cache_manager = self.settings.get('cache_manager', None)
        if not s_cache_manager:
            raise CacheError('use cache , app\'s settings must has cache_manager')
        if on == None: # 表示获取默认的cache manager
            if isinstance(s_cache_manager, (tuple, list)): # 如果有指定多个，则默认是以第一个为cache_manager
                s_cache_manager = s_cache_manager[0]

            return s_cache_manager
        else:
            on = utf8(on)
            _cache_manager = getattr(self.application, on + '_cache_manager', None) # 这主要是用来减少判断的，对不同的cache_manager只需要做一个判断，就可以找出哪个on，对应的是哪个列表中的元素了
            if _cache_manager:
                return _cache_manager

            if not isinstance(s_cache_manager, (tuple, list)):
                raise CookieError("use 'on' arg, cache_manager setting must be tuple or list")
            for _cm in s_cache_manager:
                if _cm.__name__ == on:
                    setattr(self.application, on + '_cache_manager', _cm)
                    return _cm

            raise CookieError('Not Found %s cache_manager' % (on, ))

    def get_401_user_pwd(self):
        """在401验证时，获取用户输入的用户名和密码，返回tuple"""
        authorization = self.request.headers.get('Authorization')
        if not authorization:
            return None, None
        import base64
        user_pwd = base64.decodestring(authorization.split()[-1].strip())
        return user_pwd.split(':', 1)

    def on_connect_close(self):
        pass

class _FileItem(object):
    def __init__(self, root, relative_path):
        self.root = root
        self.relative_path = relative_path or '/'

    @property
    def pretty_name(self):
        basename = self.relative_path
        if self.isdir and self.relative_path[-1] != '/':
            return basename + '/'
        else:
            return basename

    @property
    def showpath(self):
        return os.path.join(self.root, self.relative_path)

    @property
    def dirname(self):
        return os.path.dirname(self.abspath)

    @property
    def isfile(self):
        return os.path.isfile(self.abspath)

    @property
    def isdir(self):
        return os.path.isdir(self.abspath)

    @property
    def name(self):
        return os.path.basename(self.abspath)

    @property
    def ishidden(self):
        basename = os.path.basename(self.abspath)
        return basename[0] == '.'

    def __repr__(self):
        return self.abspath

    @property
    def raw_size(self):
        return self.stat.st_size

    @property
    def pretty_size(self):
        raw_size = self.raw_size
        pretty_size = raw_size

        unit = 'B'
        for _unit in ['KB', 'MB', 'GB']:
            pretty_size /= 1024.0
            if pretty_size <= 1000:
                unit = _unit
                break

        return '%.2f %s' % (pretty_size,
                unit)

    @property
    def stat(self):
        return os.stat(self.abspath)

    @property
    def ctime(self):
        return time.ctime(self.stat.st_ctime)

    @property
    def mtime(self):
        return time.ctime(self.stat.st_mtime)

    @property
    def isexist(self):
        return os.path.exists(self.abspath)

    @property
    def abspath(self):
        if self.relative_path[0] == '/':
            path = self.relative_path[1:]
        else:
            path = self.relative_path

        abspath = os.path.join(self.root, path)
        return abspath

class RedirectHandler(RequestHandler):
    def GET(self):
        self.redirect(**self.kwargs)

class FileHandler(RequestHandler):
    """
    列出所有本地所有文件及目录
    kwargs
        root: 本地文件目录
        show_hidden: 显示隐藏文件(默认False)
        hidden_list: 隐藏部分文件(正则表达式)
        deny_list: 禁止部分文件访问(正则表达式)
        deny_hidden: 禁止hidden_list的文件
        base_username: 401用户名
        base_password: 401密码
    """
    def init_data(self):
        hidden_list = self.kwargs.get('hidden_list', [])
        deny_list = self.kwargs.get('deny_list', [])

        if self.kwargs.get('deny_hidden'):
            deny_list.extend(hidden_list)

        self.hidden_re_list = [ re.compile(hidden_exp) for hidden_exp in hidden_list]
        self.deny_re_list = [re.compile(deny_exp) for deny_exp in deny_list]
        self.base_username = self.kwargs.get('base_username')
        self.base_password = self.kwargs.get('base_password')

    def GET(self, relative_path = '/'):
        if self.base_username and self.base_password:
            auth_401(self.__class__._GET)(self, relative_path)

        else:
            self._GET(relative_path)

    def _GET(self, relative_path):
        relative_path = relative_path or '/'
        if relative_path.startswith('./'):
            self.raise_error(403)

        self.relative_path = relative_path

        item = _FileItem(self.root, relative_path)
        if not (item.isexist and self.allow_account(item)):
            self.raise_error(404)

        elif item.isfile:
            self.set_header('Content-Type', get_mime_type(item.abspath))
            self.send_file(item.abspath, is_attr = self.query.attr == '1')
        else:
            self.render('files.html', items = self.ls(item.dirname), 
                    relative_path = relative_path, parent = item.dirname)

    def get_current_user(self):
        username, password = self.get_401_user_pwd()
        if username == self.base_username and password == self.base_password:
            return username

        else:
            return None

    def allow_account(self, item):
        item_urlpath = os.path.join(self.relative_path, item.pretty_name)
        return not any([deny_re.search(item_urlpath) for deny_re in self.deny_re_list])

    def allow_show(self, item):
        allow_hiddent = self.kwargs.get('show_hidden', False)
        if (not allow_hiddent) and item.ishidden:
            return False

        item_urlpath = os.path.join(self.relative_path, item.pretty_name)
        return not any([hidden_re.search(item_urlpath) for hidden_re in self.hidden_re_list])

    def ls(self, dir_path):
        items = []

        for item_name in os.listdir(dir_path):
            new_item = _FileItem(dir_path, item_name)


            if not self.allow_show(new_item):
                continue

            items.append(new_item)

        return items

    @property
    def root(self):
        return self.kwargs.get('root', '/')

    def get_template_path(self):
        return self._generate_abspath('template')

class ErrorHandler(RequestHandler):
    def ALL(self):
        status_code = self.kwargs.get('status_code', 500)
        raise HTTPError(status_code)

class Application(object):
    _buffer_stringio = s_io()
    _template_cache = {}

    def __new__(obj, *args, **kwargs):
        if hasattr(Application, '_instance'):
            return Application._instance

        _instance = object.__new__(obj, *args, **kwargs)
        Application._instance = _instance

        return _instance

    def __init__(self, handlers = [], vhost_handlers = [], settings = {}, log_settings = {},  ui_settings = {}):
        """
        log_settings : {'level': log level(default: DEBUG',
                'datefmt': log date format(default: "%Y-%m-%d %H:%M:%S"),
                'file': log save to file}
        ui_settings: {module_name: module(extends from UIModule)}
        """
        self.settings = settings
        self.ui_settings = ui_settings
        default_handlers = []

        if settings.get('static_path'):
            static_class = settings.get('static_class', StaticFileHandler)
            for url_re in (r'%s(.*)' %
                (settings.get('static_prefix', r'/static/')),
                r'/(favicon\.ico)', r'/(robots\.txt)'):
                default_handlers.append((url_re, static_class))

        gale_static_path = os.path.join(os.path.dirname(os.path.abspath(__file__)),
                'static')

        if settings.get('debug'):
            _debug_prefix = settings.get('debug_prefix', '/')
            _debug_handler_url = os.path.join(_debug_prefix, 'gale/debug')
            _debug_static_url = os.path.join(_debug_prefix, 'gale/static')

            default_handlers.append((_debug_handler_url, DebugHandler, 
                {'static_url_path': _debug_static_url}))

            default_handlers.append((r'%s/(.*)' % (_debug_static_url),
                StaticFileHandler, {'root_path': 
                    gale_static_path}))

        if 'session_manager'  in settings:
            self.session_manager = settings['session_manager']

        vhost_handlers = vhost_handlers or [('.*$', handlers)]
        self.vhost_handlers = []
        for _vhost_handler in vhost_handlers:
            self.add_handlers(*_vhost_handler)

        # 缺省的一些handler被从vhost_handlers中分离出来, 使缺省的handler应用到所有的host handlers中去
        self.default_handlers = []
        for default_handler in default_handlers:
            if len(default_handler) == 2:
                _url, _hdl = default_handler
                self.default_handlers.append([self.__re_compile(_url),
                    _hdl, {}])

            elif len(default_handler) == 3:
                _url, _hdl, _kwargs = default_handler
                self.default_handlers.append([self.__re_compile(_url),
                    _hdl, _kwargs])

        config_logging(log_settings)

    def __re_compile(self, re_string):
        """把正则规则都编译了，加快速度"""
        re_string = "%s%s%s" % ((not re_string.startswith('^')) and '^' or '', 
            re_string, (not re_string.endswith('$')) and '$' or '')  # 自动给url加上开头与结尾
        return re.compile(re_string)

    def add_handlers(self, re_host_string, host_handlers, to_handlers = None):
        to_handlers = self.vhost_handlers if to_handlers == None else to_handlers
        _re_host_exists = False
        _compile_handlers = []
        for _a_url_item in host_handlers:
            assert len(_a_url_item) in (2, 3), "url args length must be 2 or 3 "

            if len(_a_url_item) == 2: # 如果没有传入handler参数，默认设定是空
                _a_url_item += ({}, )
            _re_string, _handler, _kwargs = _a_url_item
            _compile_handlers.append((self.__re_compile(_re_string), 
                _handler, _kwargs))

        for _re, _handlers in self.vhost_handlers:
            if _re.pattern == re_host_string:
                _handlers.extend(_compile_handlers)
                _re_host_exists = True
                break
        if not _re_host_exists:
            to_handlers.insert(-1, (self.__re_compile(re_host_string), _compile_handlers))
    
    def __call__(self, request, is_wsgi = False, server_settings = None):
        if not request: # 如果无法获取一个request，则结束它，表示连接已经断开
            return

        self.server_settings = server_settings or {}

        request.real_ip = self.settings.get('real_ip', True) # 设定是否要获取真实ip地址

        handler, url_args, url_kwargs = self.__find_handler(request)
        url_args = map(urlunquote, url_args)
        try:
            self.__exec_request(handler, request, *url_args, **url_kwargs)
        except Exception as e:
            handler.process_raise_error(e) # 把异常传入，并分析，执行错误处理方法(push_error)
            if handler.get_status() >= 500: # 只有错误代码大于等于500才会打印出异常信息
                traceback.print_exc()
        finally:
            if (not is_wsgi) and (getattr(handler, '_is_auto_finish', False) == False):
                handler.finish()

        return handler


    def __exec_request(self, handler, request, *args, **kwargs):
        handler.ALL(*args, **kwargs) #  所有请求前都要先执行它
        _method_func = getattr(handler, request.method, None) 
        if not _method_func:
            handler.set_header('Allow', 
                    ', '.join(handler.allow_methods))
            raise NotSupportMethod

        _method_func(*args, **kwargs)

    def __find_handler(self, request):
        """根据url来决定将任务交由哪个handler去处理, 会返回handler，还有url参数"""
        request_path = request.path
        request_host = request.host

        for vhost_re, handlers in self.vhost_handlers: # 如果启用了虚拟主机，则做一下host字段的匹配
            if vhost_re.match(request_host):
                self.handlers = handlers
                break

        for url_re, url_handler, kwargs in self.handlers + self.default_handlers:
            _match = url_re.match(request_path)
            if _match: # 如果匹配上了，就执行下一步
                return url_handler(self, request, kwargs = kwargs), _match.groups(), _match.groupdict()

        default_handler = self.settings.get('default_handler')
        if default_handler: # 如果指定了默认处理，则调用，否则让ErrorHandler处理它，也就是会被当404处理
            return default_handler(self, request), (), {}
        else:
            return ErrorHandler(self, request, kwargs = {'status_code': 404}), (), {}


    def router(self, url, method = None, base_handler = None, host = '^.*$', kwargs = None, is_login = False, should_login = False, bind_methods = None, handler = None):
        assert not (is_login and should_login) # 同时是登录类，又需要登录，这怎么可能呢

        bind_methods = bind_methods or {}

        base_handler =  handler or self.__made_base_handler(host, url, base_handler) # 根据url来生成一个类

        for method_name, method_instance in bind_methods.items():
            setattr(base_handler, method_name, method_instance)


        method = method or 'GET'
        kwargs = kwargs or {}

        if handler:
            self.add_handlers(host, [(url, handler, kwargs), ])
            return

        if is_login:
            self.settings['login_url'] = url

        if not isinstance(method, (tuple, list)):
            method = [method, ]

        for _method in method:
            if _method.upper() not in _ALL_METHOD:
                raise NotSupportMethod

        def method_func_wrap(method_func):
            @wraps(method_func)
            def wrap(self, *args, **kwargs):
                if should_login:
                    _func = authenticated(method_func)
                else:
                    _func = method_func

                return _func(self, *args, **kwargs)

            for _method in method:
                setattr(base_handler, _method.upper(), wrap)

            return wrap

        host_handler = (url, base_handler, kwargs)
        """ 这里需要处理把base添加到原来的vhost_handlers中去，如果已经有.*$了，并且有指定 host，就插入新的，如果没有host指定，则追进到旧的里去"""
        self.add_handlers(host, [host_handler, ])

        return method_func_wrap

    def __made_base_handler(self, host, url, base_handler = None):
        _base_handler = None
        url_compile = self.__re_compile(url)
        for re_host, vhost_handlers in self.vhost_handlers: # 先遍历下，看看有没有已经存在了的类
            if (re_host.match(host)):
                for re_url, _handler, _kwargs in vhost_handlers:
                    if re_url.pattern == url_compile.pattern:
                        _base_handler = _handler
                        vhost_handlers.remove((re_url, _handler, _kwargs))
                        break
                continue

        if _base_handler:
            return _base_handler

        _m = hashlib.md5(utf8(url))
        _class_name = '_T%s_Handler' % (_m.hexdigest())
        return type(native_str(_class_name),
                (base_handler or RequestHandler, ), {})

    def run(self, host = '', port = 8080, processes = 0, **server_settings):
        http_server = HTTPServer(self, 
                host = host, port = port,
                **server_settings)
        http_server.run(processes)

    @property
    @cache.cache_self
    def _static_md5_cache(self):
        if self.server_settings['processes'] == 1:
            return {}
        else:
            return IPCDict('_static_md5_cache')

def limit_referrer(method):
    """防外链"""
    @wraps(method)
    def wrap(self, *args, **kwargs):
        referrer = self.request.get_header('referrer')
        referrer_host = referrer and urlsplit(referrer).netloc.split(':')[0]
        if (not referrer) or referrer_host != self.request.host:
            self.on_referrer_error()
            return

        return method(self, *args, **kwargs)
    
    return wrap

def authenticated(method):
    @wraps(method)
    def wrap(self, *args, **kwargs):
        if self.current_user:
            return method(self, *args, **kwargs)
        else:
            login_url = self.login_url
            if not login_url:
                raise LoginHandlerNotExists('Not found login handler or "login_url" not in application settings')
            if self.request.method in ('GET', 'HEAD'):
                callback_url = urlquote(self.request.uri)
                url_joiner = '?' in login_url and  '&'  or '?'
                self.redirect(login_url + url_joiner + 'callback=' + callback_url)
            else:
                raise HTTPError(403)

    return wrap

def async(method):
    @wraps(method)
    def wrap(self, *args, **kwargs):
        self._is_auto_finish = True
        return method(self, *args, **kwargs)

    return wrap


class UIModule(object):
    def __init__(self, handler):
        self.handler = handler
        self.request = handler.request

    def render(self):
        raise NotImplementedError

    def render_string(self, template_name, *args, **kwargs):
        return self.handler.render_string(template_name, *args, **kwargs)

    def string_render(self, string, *args, **kwargs):
        return self.handler.string_render(string, *args, **kwargs)


class StaticFileHandler(RequestHandler):
    absolute_path = None
    def HEAD(self, file_path):
        return self.GET(file_path, is_include_body = False)

    def get_static_root_path(self):
        static_root_path = self.kwargs.get('root_path') or self.static_path

        return static_root_path

    def GET(self, file_path, is_include_body = True):
        self.__is_sending_file = True
        absolute_path = self.make_absolute_path(self.get_static_root_path(),
                file_path)
        if not os.path.exists(absolute_path):
            raise HTTPError(404)

        self.absolute_path = absolute_path

        self.set_header('Content-Type', get_mime_type(self.absolute_path))

        if self.get_content_size() < 1024 * 1024 * 10: # 10m以下的文件才做缓存
            content_version = self.get_content_version(absolute_path)

            self.set_expire_header()
            self.set_etag_header(content_version)

            is_return_304 = self.should_return_304(content_version)
        else:
            is_return_304 = False

        if is_return_304:
            self.set_status(304)
        else:
            self.set_header('Content-Length', self.get_content_size())

        if is_include_body and (not is_return_304): # 如果是Flase，则表示是HEAD方法传过来的，这时候不需要发送文件
            for _data in self.get_content(absolute_path):
                self.push(_data)
                self.flush()

    def get_content_size(self):
        return self.file_stat().st_size

    def should_return_304(self, content_version):
        """判断是否需要返回304,判断下request中的内容hash值与计算出来的是否一样就行了"""
        cache_control = self.request.get_header('Cache-Control')
        if cache_control == 'no-cache':
            return False
        return self.get_request_version() == content_version

    def get_request_version(self):
        """获取request中的内容hash值"""
        version = self.request.headers.get('If-None-Match', '')
        return version

    def process_buffer(self, _buffer):
        return _buffer

    @classmethod
    def get_content(cls, absolute_path):
        with open(absolute_path, 'rb') as fd:
            while True:
                chunk_size = 1024 * 8
                chunk = fd.read(chunk_size)
                if chunk:
                    yield chunk
                else:
                    return


    def set_expire_header(self):
        """这是在设置跟过期有关的http头"""
        expire_sec = 86400 * 365 * 10
        if self.is_supported_http1_1():
            self.set_header('Cache-Control', 'max-age=%s' % (expire_sec))
        else:
            self.set_header('Expires', format_timestamp(time.time() + expire_sec))


    def get_content_version(self, absolute_path):
        _cache_version = self.get_cache_version(absolute_path)
        if _cache_version:
            return _cache_version

        md5er = hashlib.md5()
        for _chunk in self.get_content(absolute_path):
            md5er.update(_chunk)
        _md5 = md5er.hexdigest() 
        self._md5_cache[absolute_path] = {'md5': _md5,
                'mtime': self.file_stat().st_mtime}

        return _md5

    def get_cache_version(self, absolute_path):
        _cache = self._md5_cache.get(absolute_path)
        if not _cache:
            return None

        if self.file_stat().st_mtime != _cache['mtime']: # 如果已经修改过了，则表示已经更新了，则需要重新计算hash值
            del self._md5_cache[absolute_path]
            return None

        return _cache['md5']

    @property
    def _md5_cache(self):
        return self.application._static_md5_cache

    @classmethod
    def make_absolute_path(cls, root, path):
        return os.path.join(root, path)

    @classmethod
    def get_static_url(cls, settings, file_path):
        static_path = settings.get('static_path')
        if static_path.startswith('http'): # 让static同时支持在线资源
            return urlparse.urljoin(static_path, file_path)

        static_prefix = settings.get('static_prefix', '/static/')
        return urlparse.urljoin(static_prefix, file_path)

    def file_stat(self):
        return os.stat(self.absolute_path)

    def set_etag_header(self, etag = None):
        if not self.is_supported_http1_1(): # 如果不是http 1.1则是不支持的（比如 1.0)
            return 

        etag = etag or self.get_content_version(self.absolute_path)
        self.set_header('Etag', etag)


class GzipProcessor(object):
    COMPRESSION_LEVEL = 7
    ALLOW_CONTENT_TYPE = ('text/*', '*javascript', '*json*', '*xml*', 
            'image/*')
    MIN_LENGTH = 100

    def __init__(self, request_headers):
        self.should_gzip = 'gzip' in request_headers.get('Accept-Encoding', '')

    def process(self, _buffer, response_headers, _buffer_stringio):
        if len(_buffer) < self.MIN_LENGTH:
            self.should_gzip = False

        if not self.should_gzip:
            return _buffer

        is_compesssion_type = False
        content_type = response_headers.get('Content-Type', '').split(';')[0]
        if not content_type: # 如果没有指定 Content-Type则返回 
            return _buffer
        for _type in self.ALLOW_CONTENT_TYPE:
            if glob.fnmatch.fnmatch(content_type, _type):
                is_compesssion_type = True
                break

        if not is_compesssion_type:  # 如果不是压缩对象，则也返回
            return _buffer


        _gzip_file = gzip.GzipFile(mode = 'wb', fileobj = _buffer_stringio, compresslevel = self.COMPRESSION_LEVEL)
        _gzip_file.write(_buffer)
        _gzip_file.close()
        compression_data  = _buffer_stringio.getvalue()
        _buffer_stringio.seek(0)
        _buffer_stringio.truncate()

        response_headers['Content-Encoding']  = 'gzip'

        return compression_data

class _UINameSpace(object):
    def __init__(self, handler):
        self.handler = handler
    
    def __getattr__(self, module):
        _module = self.handler.ui_settings.get(module)
        _module_instance = _module(self.handler)
        return _module_instance.render

def create_cookie_signature(secret, value_b64encoded):
    hash_obj = hmac.new(utf8(secret), value_b64encoded, digestmod =  hashlib.sha256)
    return utf8(hash_obj.hexdigest()).upper()

def encode_signed_cookie(secret, name, value):
    name = utf8(name)
    value = utf8(value)
    value = base64.b64encode(value)
    signed_secret = create_cookie_signature(secret, value)
    signed_cookie = "|%s:%s|%s:%s%s" %(
            len(name), name, len(value), value, signed_secret)

    return signed_cookie

def decode_signed_cookie(secret, name, value):
    assert value[0] == '|'
    signed_cookie = value
    def split_signed_cookie(rest):
        offset = rest.find(':')
        field_length = int(rest[1: offset])
        field_value = rest[offset + 1: offset + field_length + 1]

        rest = rest[field_length + offset + 1: ]

        return field_value, rest

    try:
        field_name, rest = split_signed_cookie(signed_cookie)
        field_value, rest = split_signed_cookie(rest)
    except ValueError:
        return None


    if create_cookie_signature(secret, field_value) != rest:
        return None

    return base64.b64decode(field_value)

def decode_signed_cookie_tornado(secret, name, value):
    signed_cookie = value
    rest = signed_cookie[2:]
    def split_signed_cookie(rest):
        offset = rest.find(b':')
        length = int(rest[0: offset])
        field_value = rest[offset + 1: length + offset + 1]

        rest = rest[length + offset + 2:]

        return field_value, rest

    key_version, rest = split_signed_cookie(rest)
    timestamp, rest = split_signed_cookie(rest)
    field_name, rest = split_signed_cookie(rest)
    field_value, rest = split_signed_cookie(rest)

    if int(timestamp) < int(time.time()) - 30 * 86400:
        return None

    return base64.decodestring(field_value)

class ParamsObject(object):
    def __init__(self, params):
        self._params =  params

    def __getitem__(self, name):
        value = self._params.get(name, None)
        if value == None:
            return None

        return value[0] if len(value) == 1 else value

    def __getattr__(self, name):
        return self[name]

    def __repr__(self):
        return json.dumps(self._params, indent = 4)

class BodyParamsResponseObject(ParamsObject):
    def __init__(self, params):
        self._buffer = []
        super(BodyParamsResponseObject, self).__init__(params)

    def append_buffer(self, _buffer):
        self._buffer.append(utf8(_buffer))

    def set_buffer(self, _buffer):
        self._buffer = []

        if not _buffer:
            self.append_buffer(_buffer)

    def empty(self):
        return self._buffer == []

    def to_bytes(self):
        return b''.join(self._buffer)

    def __add__(self, _buffer):
        self.append_buffer(_buffer)

    def flush_buffer(self):
        self._buffer = []

class DebugHandler(RequestHandler):
    def ALL(self):
        if not self.settings.get('debug'):
            raise HTTPError(404)

        self.restapi = RestApi(self.application.vhost_handlers)

    def GET(self):
        restapi_list = self.restapi.generate_restapi_list()

        api_module_tree = list(restapi_list.keys())
        self.render('doc.html', api_description = '', api_module_tree = api_module_tree,
                api_docs = restapi_list, 
                cat_response_params = self._cat_response_params,
                cat_request_params = self._cat_request_params)


    def get_static_url(self, file_path):
        static_url_path = self.kwargs['static_url_path']
        return os.path.join(static_url_path, file_path)

    def _cat_response_params(self, response_params):
        response_params = response_params or {}
        public_response_params = self.settings.get('restapi_public_response', {})
        response_params.update(public_response_params)

        return response_params

    def _cat_request_params(self, request_params):
        if not request_params:
            return {}
        public_request_params = self.settings.get('restapi_public_request', {})
        request_params.update(public_request_params)
        return request_params


    def get_template_path(self):
        return self._generate_abspath('template')

from gale.server import HTTPServer

@single_pattern
class _HANDLER_LIST(list):
    pass

HANDLER_LIST = _HANDLER_LIST()

def router(*args, **kwargs):
    def wraps(method_func = None):
        HANDLER_LIST.append((method_func, args, kwargs))

    handler = kwargs.get('handler')

    if handler:
        wraps()
    else:
        return wraps

def app_run(app_path = None, settings = {}, log_settings={}, server_settings = {}, host = '', port = 8080, processes = 0):
    """在这里的app_path决定着默认template和static_path，默认是执行脚本程序时的工作目录"""
    settings.setdefault('debug', True)
    settings.setdefault('static_path', os.path.join(app_path and os.path.dirname(os.path.abspath(app_path)) or os.getcwd(), 'static'))
    settings.setdefault('template_path', os.path.join(app_path and os.path.dirname(os.path.abspath(app_path)) or os.getcwd(), 'template'))

    app = Application(settings = settings, 
            log_settings = log_settings)

    for handler_func, args, kwargs in HANDLER_LIST:
        if handler_func:
            app.router(*args, **kwargs)(handler_func)
        else:
            app.router(*args, **kwargs)

    http_server = HTTPServer(app, host = host, port = port, **server_settings)
    http_server.run(processes = processes)

